/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.audiocd;

import javax.swing.JComponent;

import net.sf.dc.console.components.panels.TableSearchPanel;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.FillTableRequest;

/**
 * The Audio CD search panel
 * 
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class SearchAudioCDPanel extends TableSearchPanel {

    public SearchAudioCDPanel() {
        super(DCModules.getModule(DCModules._AUDIOCD),
              DcResources.getText(Repository.systemlabels.sysAudioCDPlural), 
              IconLibrary._icoExisting);
    }
    
    public JComponent[] getExtraComponents() {
        return new JComponent[0];
    }
    
    /**
     * Tells the child panel to load children for the current row
     */
    public void loadChildValues() {
        if (hasChild()) {
            DcObject dcoChild = getChildModule().getDCObject();
            int selectedRow = table.getSelectedRow();
            
            if (selectedRow > -1) {
                // Pass the ID of the parent to the child panel
                String ID = table.getObjectID(selectedRow);
                childPanel.setParentID(ID);
                
                dcoChild.setValue(dcoChild.getParentReferenceFieldIndex(), ID);
                
                // Ask the child panel to load and display the data
                childPanel.clearTable();
                
                DCModule childModule = getChildModule();
                dcoChild.addRequest(new FillTableRequest(childModule.getIndex()));

                DcField  field = childModule.getField(AudioTrack._F_TRACKNUMBER);
                QueryOptions options = new QueryOptions(field, true, true);
                dcoChild.load(options);
            }
        }
    }    
    
    public void removeNewChild (String sID) {}
}
