/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

/**
 * Simple HSB color field derived from the JColorChooser field.
 * Applies the color to the application everytime a new one has 
 * been selected 
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class DcColorSelector extends JComponent implements ISettingsComponent {
    
    private JPanel panelExample = new JPanel(false);
    private JPanel panelChooser = new JPanel(false);
    
    private ColorSelectionModel colorModel;
    private Color color;
    
    private DcLongTextField textExample = ComponentFactory.getLongTextField(400);
    
    private AbstractColorChooserPanel panelHSB = new JColorChooser().getChooserPanels()[1];
    private boolean applicationColor = false;
    private String settingsKey = "";

    public DcColorSelector(boolean applicationColor, String settingsKey) {
        this.applicationColor = applicationColor;
        this.settingsKey = settingsKey;
        buildComponent();
    }
    
    public Object getValue() {
        return color;   
    }

    public void setValue(Object o) {
        color = (Color) o;
        colorModel.setSelectedColor(color);
        panelHSB.updateChooser();
        textExample.setBackground(color);
    }
    
    /**
     * Builds this component. Basically it takes the JColorChooser 
     * field apart and builds this component.
     */
    public void buildComponent() {
        this.setLayout(Layout.getGBL());
        
        JLabel labelExample = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblPreview));
        
        
        // get the components from the JColorChooser
        JPanel panel = (JPanel) panelHSB.getComponent(0);
        colorModel = panelHSB.getColorSelectionModel();
        Component[] comps2 = panel.getComponents();
        
        // add them to this components
        for (int j = 0; j < comps2.length; j++) {
            panelChooser.add(comps2[j]);
        }
        
        panelChooser.setMaximumSize(new Dimension(300,200));
        panelChooser.setPreferredSize(new Dimension(300,200));

        textExample.setBorder(ComponentFactory.getTitleBorder(""));

        panelExample.setLayout(Layout.getGBL());
        panelExample.add(labelExample, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                         GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0,0,0,0), 0, 0));
        panelExample.add(textExample, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                         GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						 new Insets(0,0,0,0), 0, 0));
        textExample.setText("data crow DATA CROW Data Crow dATA cROW \r\n" +
                            "ndata crow DATA CROW Data Crow dATA cROW ");
        
        add(panelChooser, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0,0,0,0), 0, 0));
        add(panelExample, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0,0,0,0), 0, 0));

        comps2 = null;
        
    	colorModel.addChangeListener(new StateChanged());
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        panelExample.setBackground(color);
        panelChooser.setBackground(color);
    }
    
    /**
     * Hooks directly into the applications UI. Updates the 
     * background color of the UI when the user selects a different
     * color
     */
    private class StateChanged implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            Color temp = colorModel.getSelectedColor();
            if (!color.equals(temp)) {
                color = temp;
                textExample.setBackground(color);
                if (applicationColor) {
                	DataCrow.mainFrame.setBackgroundColor(color);
                } else if (settingsKey != null) {
                    DcSettings.setValue(settingsKey, color);
                }
            }
        }
    }
}
