/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

/**
 * @version 2.0
 * @since 1.4
 * @author Robert Jan van der Waals
 */
public class DcFieldSelectorField extends JComponent implements ISettingsComponent {

    private final Map componentMap = new HashMap();
    private final int module; 
    
    public DcFieldSelectorField(int module) {
        this.module = module;
        buildComponent();        
    }
    
    public Object getValue() {
        return getEnabledFields();
    }
    
    public void setValue(Object o) {
        if (o instanceof int[]) {
            setSelectedFields((int[]) o);
        }
    }

    public Collection getEnabledFields() {
        Collection fields = new ArrayList();
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);

            if (checkBox.isSelected()) 
                fields.add(field);
        }
        
        return fields;
    }
    
    public void setSelectedFields(int[] fields) {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            
            if (fields.length == 0) {
                checkBox.setSelected(!field.isHidden());
            } else {
                for (int j = 0; j < fields.length; j++) {
                    if (field.getIndex() == fields[j]) 
                        checkBox.setSelected(true);
                }
            }
        }
    }
    
    private void buildComponent() {
        setLayout(Layout.getGBL());
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        JPanel panel = new JPanel(false);
        panel.setLayout(Layout.getGBL());
        panel.setBackground(color);
        
        DcObject dco = DCModules.getModule(module).getDCObject();
        
        int x = 0;
        int y = 0;
        
        Collection fields = dco.getFields();
        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            
            JCheckBox checkBox = ComponentFactory.getCheckBox(field.getLabel());
            checkBox.setBackground(color);
            componentMap.put(field, checkBox);
            
            panel.add(checkBox, Layout.getGBC(x++, y, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets( 0, 10, 0, 0), 0, 0));            
            
            if (x == 4) {
                x = 0;
                y++;
            }
        }
        
        add(panel, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                   new Insets( 0, 0, 0, 0), 0, 0));            
        
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        Component[] comps = getComponents();
        for (int i = 0; i < comps.length; i++) {
            JComponent component = (JComponent) comps[i];
            component.setBackground(color);
        }
        
        for (Iterator iter = componentMap.values().iterator(); iter.hasNext(); ) {
            ((JComponent) iter.next()).setBackground(color);
        }
    }      
}
