/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.AboutAction;
import net.sf.dc.core.actions.CreateNewAction;
import net.sf.dc.core.actions.CreateReportAction;
import net.sf.dc.core.actions.DeleteAction;
import net.sf.dc.core.actions.ExpertUserAction;
import net.sf.dc.core.actions.HelpAction;
import net.sf.dc.core.actions.InternetUpdateAction;
import net.sf.dc.core.actions.LogAction;
import net.sf.dc.core.actions.NewItemWizardAction;
import net.sf.dc.core.actions.OpenAction;
import net.sf.dc.core.actions.OpenBackupDialogAction;
import net.sf.dc.core.actions.OpenFieldsSettingsDialog;
import net.sf.dc.core.actions.OpenModuleAction;
import net.sf.dc.core.actions.OpenResourceEditorAction;
import net.sf.dc.core.actions.QuitApplicationAction;
import net.sf.dc.core.actions.SettingsAction;
import net.sf.dc.core.actions.StartProgramAction;
import net.sf.dc.core.actions.ToggleModuleListAction;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.actions.ToggleToolbarLabelsAction;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.resources.DcResources;

/**
 * Creates a menu bar for a specific module (dynamic). 
 * All actions for this menubar 
 * can be found in the net.sf.dc.core.actions package
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class DcMenuBar extends JMenuBar {

	public static final int _SEARCHPANEL = 0;
    public static final int _INSERTPANEL = 1;
    public static final int _NOTEPANEL = 2;
    
    private final DCModule module;

    /**
     * Initializes this menubar for the specified module
     * @param module
     */
    public DcMenuBar(DCModule module) {
        this.module = module;
    	build();
    }
    
    public void setPanel(int index) {
        switch (index) {
            case _SEARCHPANEL:
                break;
            case _INSERTPANEL:
                break;
            case _NOTEPANEL:
                break;
        }
    }
    
    /**
     * Builds this menubar
     */
    private void build() {
        String dcoName = module.getObjectName();
        
        JMenu menuTools = module.getToolsMenu();
        if (menuTools == null) {
        	menuTools = ComponentFactory.getMenu(
                    DcResources.getText(Repository.labels.lblTools));
        }
        
        JMenu menuSettings = ComponentFactory.getMenu(
                DcResources.getText(Repository.labels.lblSettings));
        JMenu menuFile = ComponentFactory.getMenu(
                DcResources.getText(Repository.labels.lblFile));
        JMenu menuInformation = ComponentFactory.getMenu(
                DcResources.getText(Repository.labels.lblInformation));

        // information menu
        JMenuItem menuItemAbout = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuAbout16,
                DcResources.getText(Repository.labels.lblAbout));
        JMenuItem menuItemHelp = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuHelp16, 
                DcResources.getText(Repository.labels.lblHelp));

        menuItemHelp.addActionListener(new HelpAction());
        menuItemAbout.addActionListener(new AboutAction());

        menuInformation.add(menuItemAbout);
        menuInformation.add(menuItemHelp);
        
        // field menu
        JMenuItem menuNewItemWizard = ComponentFactory.getMenuItem(     
                IconLibrary._icoMenuWizard16, 
                "New Item Wizard");
        
        JMenuItem menuItemLog = ComponentFactory.getMenuItem(
                DcResources.getText(Repository.labels.lblLogEvent));
        JMenuItem menuItemNew = ComponentFactory.getMenuItem(     
                IconLibrary._icoMenuNew16, 
                DcResources.getText(Repository.labels.lblNewItem, dcoName));
        JMenuItem menuItemOpen = ComponentFactory.getMenuItem(    
                IconLibrary._icoMenuOpen16,
				DcResources.getText(Repository.labels.lblOpenItem, dcoName));
        JMenuItem menuItemDelete = ComponentFactory.getMenuItem(  
                IconLibrary._icoMenuRemove16, 
                DcResources.getText(Repository.labels.lblDeleteItem, dcoName));
        JMenuItem menuItemQuit = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuExit16,
				DcResources.getText(Repository.labels.lblQuit));
        JMenuItem menuItemLaunch = ComponentFactory.getMenuItem(
                IconLibrary._icoPlay16,
                DcResources.getText(Repository.labels.lblStartProgram));
        

        menuItemNew.addActionListener(new CreateNewAction());
        menuItemOpen.addActionListener(new OpenAction());
        menuItemQuit.addActionListener(new QuitApplicationAction());
        menuItemDelete.addActionListener(new DeleteAction());
        menuItemLog.addActionListener(new LogAction());
        menuNewItemWizard.addActionListener(new NewItemWizardAction());
        menuItemLaunch.addActionListener(new StartProgramAction());
        
        menuFile.add(menuNewItemWizard);
        menuFile.addSeparator();
        menuFile.add(menuItemLaunch);
        menuFile.addSeparator();
        menuFile.add(menuItemNew);
        menuFile.add(menuItemOpen);
        menuFile.add(menuItemDelete);
        menuFile.addSeparator();
        menuFile.add(menuItemLog);
        menuFile.add(menuItemQuit);

        // module submenu
        JMenu subMenuModule = ComponentFactory.getMenu(
                DcResources.getText(Repository.labels.lblActiveModule));
        JMenuItem menuSoftware = ComponentFactory.getMenuItem(
        		//DCModules.getModule(DCModules._SOFTWARE).getIconSmall(),
                DcResources.getText(Repository.systemlabels.sysSoftware));
        JMenuItem menuMusicFile = ComponentFactory.getMenuItem(
        		//DCModules.getModule(DCModules._MUSIC).getIconSmall(),
                DcResources.getText(Repository.systemlabels.sysMusicFile));
        JMenuItem menuMovie = ComponentFactory.getMenuItem(
        		//DCModules.getModule(DCModules._MOVIE).getIconSmall(),
                DcResources.getText(Repository.systemlabels.sysMovie));
        JMenuItem menuAudioCD = ComponentFactory.getMenuItem(
        		//DCModules.getModule(DCModules._AUDIOCD).getIconSmall(),
                DcResources.getText(Repository.systemlabels.sysAudioCD));
        JMenuItem menuBook = ComponentFactory.getMenuItem(
        		//DCModules.getModule(DCModules._BOOK).getIconSmall(),
                DcResources.getText(Repository.systemlabels.sysBook));        
        JMenuItem menuGeneric = ComponentFactory.getMenuItem(
        		//DCModules.getModule(DCModules._GENERIC).getIconSmall(),
                DcResources.getText(Repository.systemlabels.sysGenericModule));        
        
        menuSoftware.addActionListener(new OpenModuleAction(DCModules._SOFTWARE));
        menuMusicFile.addActionListener(new OpenModuleAction(DCModules._MUSIC));
        menuMovie.addActionListener(new OpenModuleAction(DCModules._MOVIE));
        menuAudioCD.addActionListener(new OpenModuleAction(DCModules._AUDIOCD));
        menuBook.addActionListener(new OpenModuleAction(DCModules._BOOK));
        menuGeneric.addActionListener(new OpenModuleAction(DCModules._GENERIC));
        
        subMenuModule.add(menuSoftware);
        subMenuModule.add(menuMovie);
        subMenuModule.add(menuMusicFile);
        subMenuModule.add(menuAudioCD);
        subMenuModule.add(menuBook);
        subMenuModule.add(menuGeneric);
        
        // settings menu
        JMenuItem menuFieldSettings = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuOption16,
                DcResources.getText(Repository.labels.lblFieldSettings));
        JMenuItem menuItemSettings = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuOption16, 
                DcResources.getText(Repository.labels.lblSettings));
        JMenuItem menuItemResources = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuOption16, 
                DcResources.getText(Repository.labels.lblResourceEditor));
        JMenuItem menuItemShowModList = ComponentFactory.getMenuItem(
                DcResources.getText(Repository.labels.lblToggleModuleList));
        JMenuItem menuToggleToolBarLabels = ComponentFactory.getMenuItem(
                DcResources.getText(Repository.labels.lblToggleToolbarLabelsAction));
        JMenuItem menuQuickView = ComponentFactory.getMenuItem(
                DcResources.getText(Repository.labels.lblToggleQuickView));
        
        menuFieldSettings.addActionListener(new OpenFieldsSettingsDialog());
        menuItemShowModList.addActionListener(new ToggleModuleListAction());
        menuItemSettings.addActionListener(new SettingsAction());
        menuToggleToolBarLabels.addActionListener(new ToggleToolbarLabelsAction());
        menuQuickView.addActionListener(new ToggleQuickViewAction());
        menuItemResources.addActionListener(new OpenResourceEditorAction());
                
        menuSettings.add(menuItemSettings);
        menuSettings.add(menuFieldSettings);
        menuSettings.add(menuItemResources);
        menuSettings.addSeparator();
        menuSettings.add(menuItemShowModList);
        menuSettings.add(menuQuickView);
        menuSettings.add(menuToggleToolBarLabels);
        menuSettings.addSeparator();
        menuSettings.add(subMenuModule);

        // tools menu
        JMenuItem menuReport = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuExport16,
                DcResources.getText(Repository.labels.lblCreatePdfReport));  
        menuReport.addActionListener(new CreateReportAction());
        
        JMenuItem menuBackup = ComponentFactory.getMenuItem(
                IconLibrary._icoBackupAndRestore,
                DcResources.getText(Repository.labels.lblBackupAndRestore));  
        menuBackup.addActionListener(new OpenBackupDialogAction());
        
        menuTools.add(menuReport);
        menuTools.add(menuBackup);
        
        if (    module.isAmazonUpdateAllowed() || 
                module.isImdbUpdateAllowed() || 
                module.getIndex() == DCModules._AUDIOCD) {

            JMenuItem menuItemOnlineSearch = ComponentFactory.getMenuItem(
                    IconLibrary._icoMenuInternetSearch16, 
                    DcResources.getText(Repository.labels.lblOnlineSearch));
            menuItemOnlineSearch.addActionListener(new InternetUpdateAction());
            menuTools.add(menuItemOnlineSearch);
        }

        JMenuItem menuExpertUser = ComponentFactory.getMenuItem(
                IconLibrary._icoMenuOption16,
                DcResources.getText(Repository.labels.lblExpertUser));
        menuExpertUser.addActionListener(new ExpertUserAction());
        menuTools.add(menuExpertUser);
        
        this.add(menuFile);
        this.add(menuSettings);

        this.add(menuTools);
        this.add(menuInformation);
    }
}
