/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTabbedPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;

/**
 * The godfather of all panels
 *
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 0.5
 */
public class DcPanel extends JPanel {

	private String helpIndex = "";
    
    private final JLabel labelStatus = new JLabel();
    private final DcProgressBar progress = new DcProgressBar();
    private final String title;
    private final ImageIcon icon;
    
    private final JPanel panelStatus = new JPanel(false);
    
    public DcPanel(String title, ImageIcon icon) {
        this.title = title;
        this.icon = icon;
        createStatusPanel();
    }
    
    /**
     * Returns the title for displaying purposes
     */
    public String getTitle() {
    	return title;
    }

    /**
     * Returns the icon belonging to this panel
     */
    public ImageIcon getIcon() {
    	return icon;
    }
    
    public void setHelpIndex(String helpIndex) {
        this.helpIndex = helpIndex;
    }
    
    public String getHelpIndex() {
    	return helpIndex;
    }

    /**
     * Updates the statusbar
     * @param text string to put in the status bar
     */
    public void setStatus(String text) {
        labelStatus.setText(text);
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        Component[] components = getComponents();
        for (int i = 0; i < components.length; i++) {
            JComponent component = (JComponent) components[i];
            setBackground(component, color);
        }
    }        
    
    private void setBackground(JComponent component, Color color) {
        if (component instanceof JLabel || component instanceof JPanel ||
            component instanceof JCheckBox || component instanceof JTabbedPane ||
            component instanceof JRootPane || component instanceof JLayeredPane) {
            
            component.setBackground(color);
            
            Component[] components = component.getComponents();
            for (int i = 0; i < components.length; i++) {
                JComponent subComponent = (JComponent) components[i];
                setBackground(subComponent, color);
            }
        }
    }    
    
    /**
     * Initializes the progress bar. Sets the maximum value
     * @param maxValue maximum value
     */
    public void initProgressBar(int maxValue) {
        progress.setValue(0);
        progress.setMaximum(maxValue);
    }
    
    /**
     * Updates the value of the progress bar
     * @param value progress value
     */
    public void updateProgressBar(int value) {
        progress.setValue(value);
    }      
    
    protected JPanel getStatusPanel() {
    	return panelStatus;
    }
    
    private void createStatusPanel() {
        panelStatus.setLayout(Layout.getGBL());
        
        labelStatus.setBorder(ComponentFactory.getTitleBorder(DcResources.getText(Repository.labels.lblStatus)));
        labelStatus.setText(Repository.system.version);

        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progress, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 0, 0, 0), 0, 0));          

        panelStatus.add(labelStatus, Layout.getGBC(0, 0, 1, 1, 20.0, 20.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 0, 0, 0), 0, 0));        
        panelStatus.add(panelProgress, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                        GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(2, 0, 2, 0), 0, 0));     
    }
}
