/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Calendar;

import javax.swing.JComponent;
import javax.swing.JLabel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

/**
 * Field for displaying hours / minutes / seconds.
 * To set the field, calculate the time in seconds
 * 
 * @author Robert Jan van der Waals
 * @version 1.5
 * @since 1.4 
 */
public class DcTimeField extends JComponent implements ISettingsComponent {
    
    private final DcNumberField fldMinutes = ComponentFactory.getNumberField();
    private final DcNumberField fldSeconds = ComponentFactory.getNumberField();
    private final DcNumberField fldHours = ComponentFactory.getNumberField();
    private final Calendar cal = Calendar.getInstance();
    
    public DcTimeField() {
    	buildComponent();
    }
    
    /**
     * Applies a value to this field
     */
    public void setValue(Object o) {
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.HOUR, 0);
        
        int value = 0;
        if (o instanceof String) {
            try {
            	value = Integer.parseInt(o.toString());
            } catch (Exception ignore) {}
        } else {
        	value = ((Integer) o).intValue();
        }

        int minutes = 0;
        int seconds = 0;
        int hours = 0;

        if (value != 0) {
        	cal.set(Calendar.SECOND, value);
        	minutes = cal.get(Calendar.MINUTE);
        	seconds = cal.get(Calendar.SECOND);
            hours = cal.get(Calendar.HOUR);
        }
        fldMinutes.setValue(minutes);
        fldSeconds.setValue(seconds);
        fldHours.setValue(hours);
    }
    
    /**
     * Retrieves the value as seconds
     */
    public Object getValue() {
    	int seconds = ((Integer) fldSeconds.getValue()).intValue();
        int minutes = ((Integer) fldMinutes.getValue()).intValue();
        int hours = ((Integer) fldHours.getValue()).intValue();
        
        int value;
        if (seconds == -1 && minutes == -1 && hours == -1) {
        	value = 0;
        } else {
            value = seconds == -1 ? 0 : seconds;
            value+= minutes == -1 ? 0 : minutes * 60;
            value+= hours == -1 ? 0 : hours * 60 * 60;
        }
        return new Integer(value);
    }
    
    public void setEnabled(boolean b) {
        super.setEnabled(b);
    	fldHours.setEnabled(b);
        fldMinutes.setEnabled(b);
        fldSeconds.setEnabled(b);
    }
    
    private void buildComponent() {
    	setLayout(Layout.getGBL());

        JLabel labelMinutes = ComponentFactory.getLabel("Minutes");
        JLabel labelSeconds = ComponentFactory.getLabel("Seconds");
        JLabel labelHours = ComponentFactory.getLabel("Hours");
        
        add(fldHours,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets( 0, 0, 0, 5), 0, 0));
        add(labelHours, Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                        new Insets( 0, 0, 0, 0), 0, 0));
        add(fldMinutes, Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets( 0, 0, 0, 5), 0, 0));
        add(labelMinutes,Layout.getGBC(3, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
					    new Insets( 0, 0, 0, 0), 0, 0));
        add(fldSeconds, Layout.getGBC( 4, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets( 0, 0, 0, 5), 0, 0));
        add(labelSeconds,Layout.getGBC(5, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
					    new Insets( 0, 0, 0, 0), 0, 0));
    }
}
