/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcModuleList;
import net.sf.dc.console.components.DcPanel;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;

/**
 * The panel for the module list.
 * 
 * @author Robert Jan van der Waals
 * @version 2.3
 * @since 1.4
 */
public class ModuleListPanel extends DcPanel {
    
    private JPanel panelHeader = new JPanel();
    private DcModuleList moduleList = new DcModuleList();

    /**
     * Initializes this component.
     */
    public ModuleListPanel() {
        super(null, null);
    	buildPanel();
    }
    
    /**
     * Sets the index to be selected in the list.
     * @param index the index to be selected
     */
    public void setSelectedIndex(int index) {
    	moduleList.setSelectedIndex(index);
    }
    
    /**
     * Creates the panel
     */
    private void buildPanel() {
        setLayout(new BorderLayout());

        JLabel label = new JLabel(DcResources.getText(Repository.labels.lblModules));
        label.setForeground(Color.BLACK);
        label.setFont(ComponentFactory.getSystemFont());
        label.setHorizontalAlignment(JLabel.CENTER);
        panelHeader.setLayout(Layout.getGBL());
        panelHeader.setBorder(BorderFactory.createEtchedBorder());
        panelHeader.add(label, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 0), 0, 0));
        
        add(panelHeader, BorderLayout.NORTH);
        add(moduleList, BorderLayout.CENTER);
		this.setBorder(new LineBorder(Color.black));
    }
}
