/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Utilities;

/**
 * @author Robert Jan van der Waals
 * @version 2.2
 * @since 1.4
 */
public abstract class QuickViewPanel extends JPanel {
    
    public static final int _DIRECTION_HORIZONTAL = 0;
    public static final int _DIRECTION_VERTICAL = 1;    
    
    protected final DCModule module;
    protected final boolean showImages;
    
    protected DcObject dco;
    private Collection images = new ArrayList();

    private JTextPane descriptionPane;
    private final JPanel panelImages = new JPanel(false);
    private final JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
    private final PopupMenu popupMenu = new PopupMenu();
    
    private final HTMLEditorKit kit = new HTMLEditorKit();
    private final HTMLDocument document = new HTMLDocument();
    
    public QuickViewPanel(DCModule module, boolean showImages) {
        this.module = module;
        this.showImages = showImages;
        setLayout(Layout.getGBL());
        
        buildPanel();
        
        changeSize();
    }    
    
    public void setObject(DcObject dco) {
        this.dco = dco;
        
        String html = "<html><body " + getBodyStyle() + ">\n" +
                        getDescriptionTable() +
                      "</body> </html>";
        
        setDescriptionHtml(html);
        descriptionPane.setCaretPosition(0);
        setImages();
    }
    
    public void clear() {
        this.dco = null;
        descriptionPane.setText("");
        panelImages.removeAll();
        
        revalidate();
        repaint();
    }
    
    private void setImages() {
        int y = 0;
        panelImages.removeAll();
        for (Iterator iter = images.iterator(); iter.hasNext(); y++) {
            String base64 = (String) iter.next();
            
            DcPictureField picField = ComponentFactory.getPictureField(true, false, false);
            picField.setMaximumSize(new Dimension(150,150));
            picField.setMinimumSize(new Dimension(150,150));
            picField.setPreferredSize(new Dimension(150,150));
            picField.setValue(base64);
            picField.setBorder(ComponentFactory.getBorder());
            
            panelImages.add(picField, Layout.getGBC(0, y, 1, 1, 1.0, 1.0,
                            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(2, 2, 2, 2), 0, 0));
        }
        panelImages.repaint();
        panelImages.revalidate();
        images.clear();
    }
    
    protected abstract String getDescriptionTable();
    
    protected void setDescriptionHtml(String html) {
        try {
            descriptionPane.setText("");
            kit.insertHTML(document, 0, html, 0, 0, null);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    protected String addTableRow(String table, int index, int textDirection) {
        if (dco.isEnabled(index)) {
            Font labelFont = DcSettings.getValueAsFont(Repository.settings.stSystemFontBold);
            Font textFont = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
            String textStyle = getStyle(textFont);
            String labelStyle = getStyle(labelFont);
    
            boolean horizontal = textDirection == _DIRECTION_HORIZONTAL;
            
            Object value = dco.getDisplayString(index);
            String realValue = dco.getValue(index) == null ? "" : dco.getValue(index).toString();
            String displayString = value == null ? "" : value.toString();
            if (realValue != null && displayString.length() > 0 && !realValue.equals("-1") && !realValue.equals("0")) {
                if (!horizontal) {
                    table += "<tr><td " + labelStyle + ">\n";
                } else {
                    table += "<tr><td " + textStyle + ">\n";
                }
                    
                table += "<b>" + dco.getLabel(index) + "</b>";
                
                if (!horizontal) {
                    table += "</td></tr>";
                    table += "<tr><td " + textStyle + ">\n";
                } else {
                    table += " ";
                }
                
                table += value;
                table += "</td></tr>";
                table += "<tr><td></td></tr>";
            } else if (dco.getField(index).getValueType() == Repository.datatypes._OBJECT) {
                String base64 = (String) dco.getValue(index);
                if (base64 != null && base64.length() > 20) 
                    images.add(base64);
            }
        }
        return table;
    }    
    
    private void buildPanel() {
        // description panel
        descriptionPane = ComponentFactory.getTextPane();
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);
        descriptionPane.setEnabled(false);
        descriptionPane.setBounds(1,1,1,1);
        descriptionPane.addMouseListener(new QuickViewMouseListener());

        JScrollPane scroller = new JScrollPane(descriptionPane);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setBorder(null);

        // images panel
        panelImages.setLayout(Layout.getGBL());
        panelImages.addMouseListener(new QuickViewMouseListener());
        
        // tabbed pane
        tabbedPane.addTab("Description", scroller);
        
        if (showImages)
        	tabbedPane.addTab("Images", panelImages);
        
        add(tabbedPane, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(0, 0, 0, 0), 0, 0));
        
        
        String html = "<html><body " + getBodyStyle() + "></body> </html>";
        setDescriptionHtml(html);
    }    
    
    private void showPopupMenu(int x, int y) {
        popupMenu.show(this, x, y);
    }    
    
    protected String getStyle(Font font) {
        int fontsize = font.getSize() > 10 ? font.getSize() : 11;
        
        String style = "style=" + '"' + "font-family:" + font.getFamily() + ";" + 
                       "font-size:" + fontsize + ";" + '"';
        return style;
    }
    
    protected String getBodyStyle() {
        
        Color colorEven = DcSettings.getValueAsColor(Repository.settings.stEvenRowColor);
        Color colorOdd = DcSettings.getValueAsColor(Repository.settings.stOddRowColor);
        
        
        String background = "";
        String foreground = "";
        if (colorEven.getRGB() < colorOdd.getRGB()) {
            background = Utilities.getHexColor(colorOdd);
            foreground = Utilities.getHexColor(colorOdd.darker().darker());
            descriptionPane.setBackground(colorOdd);
        } else {
            background = Utilities.getHexColor(colorEven);
            foreground = Utilities.getHexColor(colorEven.darker().darker());
            descriptionPane.setBackground(colorEven);
        }
        
        StringBuffer sb = new StringBuffer();
        sb.append("style=");
        sb.append('"');
        sb.append("background:;");
        
        String size = DcSettings.getValueAsString(Repository.settings.stQuickViewSize);
        int width = 205;
        if (size.equals("50%")) {
            width = width = (int) (width * 1.5);
        }
        
        String style = "style=width:" + width + ";" + '"' + "background:" + background + 
                       ";color:" + foreground + ";" + '"';
        return style;
    }

    public class PopupMenu extends JPopupMenu {
        public PopupMenu() {
            JMenuItem menuToggleLabels = new JMenuItem(DcResources.getText(Repository.labels.lblToggleQuickView));
            JMenuItem menuSize25 = new JMenuItem("25%");
            JMenuItem menuSize50= new JMenuItem("50%");
            
            menuToggleLabels.addActionListener(new ToggleQuickViewAction());
            menuSize25.addActionListener(new ToggleSize25Action());
            menuSize50.addActionListener(new ToggleSize50Action());

            this.add(menuToggleLabels);
            this.add(new JSeparator());
            this.add(menuSize25);
            this.add(menuSize50);
        }
    }   
    
    public void changeSize() {
        String size = DcSettings.getValueAsString(Repository.settings.stQuickViewSize);

        int width = 230;
        if (size.equals("50%")) {
            width = (int) (width * 1.5);
        }
        
        setMinimumSize(new Dimension(width, 10));
        setMaximumSize(new Dimension(width, 2000));
        
        if (dco != null) {
            setObject(dco);
        } else {
            String html = "<html><body " + getBodyStyle() + "></body> </html>";
            setDescriptionHtml(html);
        }
        
        revalidate();
        repaint();        
    }

    private class ToggleSize50Action implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcSettings.setValue(Repository.settings.stQuickViewSize, "50%");
            changeSize();
        }
    }
    
    private class ToggleSize25Action implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcSettings.setValue(Repository.settings.stQuickViewSize, "25%");
            changeSize();
        }
    }
    
    private class QuickViewMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                showPopupMenu(e.getX(), e.getY());
            } 

            if (e.getClickCount() == 2 && dco != null) {
            	dco.markAsUnchanged();
                ItemForm itemform = new ItemForm(null, false, true, dco, module.isImdbUpdateAllowed(), module.isAmazonUpdateAllowed());
                itemform.setVisible(true);
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
