/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Component;

import javax.swing.JComponent;
import javax.swing.JTable;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.IconCache;
import net.sf.dc.core.objects.cache.CacheManager;

/**
 * Used to show the images and text of a combo box in a table.
 * The objects displayed in a combox implementing this renderer
 * need to be cached.
 * 
 * @see net.sf.dc.core.objects.cache.Cache
 * @see net.sf.dc.core.objects.cache.CacheManager
 * 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.0
 */
public class ComboBoxTableCellRenderer extends DcTableCellRenderer {
    
    public Component getTableCellRendererComponent(
                                                JTable table, 
                                                Object value, 
                                                boolean isSelected, 
                                                boolean hasFocus, 
                                                int row,
                                                int column) {
        
        JComponent c = (JComponent) super.getTableCellRendererComponent(table,  
                                            value,
                                            isSelected,
                                            hasFocus,
                                            row,
                                            column);        

        if (value != null) {
            DcObject o = null;
			if (value instanceof DcObject) {
				o = (DcObject) value;
			} else {
                // try to retrieve the object from cache
				o = CacheManager.getObjectForDisplayValue(value.toString());
            }
            
            if (o != null) {
                // set the icon and the text
            	setIcon(IconCache.getIconForId(o.getID()));
            	setText(o.toString());
            } else {
                // else set the icon to null
                setIcon(null);
            }
            o = null;
            value = null;
        }
        return c;
    }
}
