package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolTip;
import javax.swing.filechooser.FileFilter;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcMultiLineToolTip;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.db.Database;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.Utilities;

public class BackupDialog extends DcDialog {

    private final DcFileField fileFieldTarget;
    private final DcFileField fileFieldSource;
    
    private final JButton buttonBackup = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblStart));
    private final JButton buttonRestore = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblStart));        
    
    private final JTextArea textLog = ComponentFactory.getTextArea();
    private final JProgressBar progressBar = new JProgressBar();
    
    public BackupDialog() {
        super(DataCrow.mainFrame);
        setTitle(DcResources.getText(Repository.labels.lblBackupAndRestore));
        
        BackupFileFilter filter = new BackupFileFilter();
        fileFieldSource = ComponentFactory.getFileField(false, false, filter);
        fileFieldTarget = ComponentFactory.getFileField(false, true);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        buildDialog();
    }
    
    private Collection getFiles() {
        File dataDir = new File(DataCrow.baseDir + "data" + File.separator);
        String[] list = dataDir.list();
        Collection files = new ArrayList();
        for (int i = 0; i < list.length; i++) {
            String file = list[i];
            if (!file.endsWith(".log")) {
                files.add(file);
            }
        }

        initProgressBar(files.size());
        
        return files;
    }
    
    private ZipOutputStream getZipOutputStream(String target) {
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        String date = format.format(cal.getTime());
        
        String filename = "datacrow_backup_" + date + ".bck";
        String zipFile = target.endsWith(File.separator) ? target + filename :  
                                                           target + File.separator + filename;
        ZipOutputStream zout = null;
        try {
            FileOutputStream fos = new FileOutputStream(zipFile);
            zout = new ZipOutputStream(fos);
        } catch (Exception exp) {
            new MessageBox(DcResources.getText(Repository.messages.msgBackupFileCreationError, exp.getMessage()), MessageBox._ERROR);
            Messages.add(exp);
        }
        
        return zout;
    }
    
    private void close() {
        DcSettings.setValue(Repository.settings.stBackupDialogSize, getSize());
        dispose();
    }
    
    private void addMessage(String message) {
        textLog.insert(message + '\n', 0);
        textLog.setCaretPosition(0);
    } 
    
    /**
     * Initializes the progress bar. Sets the maximum value
     * @param maxValue maximum value
     */
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }
    
    /**
     * Updates the value of the progress bar
     * @param value progress value
     */
    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }    
    
    private void restartApplication() {
        addMessage(DcResources.getText(Repository.messages.msgRestartingDb));
        WorkFlow.restartDatabase();

        addMessage(DcResources.getText(Repository.messages.msgReapplyingSettings));
        new DcSettings();
        new ReportSettings();
        new DcResources();
        MainFrame.applySettings();
        
        DataCrow.mainFrame.changeModule(DcSettings.getValueAsInt(Repository.settings.stModule));
        DCModule module = DCModules.getModule(DcSettings.getValueAsInt(Repository.settings.stModule));
        module.getSearchPanel().clearTable();
        
        DataCrow.mainFrame.setBackgroundColor(DcSettings.getValueAsColor(Repository.settings.stColor));        
    }    
    
    private void buildDialog() {
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        getContentPane().setLayout(Layout.getGBL());
        setBackground(color);
        
        
        //**********************************************************
        //Create Backup Panel
        //**********************************************************
        JPanel panelBackup = new JPanel() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
            
        panelBackup.setLayout(Layout.getGBL());
        panelBackup.setBackground(color);

        buttonBackup.addActionListener(new BackupAction());
        
        panelBackup.add(fileFieldTarget, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelBackup.add(buttonBackup,  Layout.getGBC( 1, 0, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 5, 5), 0, 0));        
        panelBackup.setToolTipText(DcResources.getText(Repository.tooltips.tpBackup));
        
        panelBackup.setBorder(
                ComponentFactory.getTitleBorder(DcResources.getText(Repository.labels.lblBackup)));
        
        //**********************************************************
        //Restore Restore Panel
        //**********************************************************
        JPanel panelRestore = new JPanel()  {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        
        panelRestore.setLayout(Layout.getGBL());
        panelRestore.setBackground(color);

        buttonRestore.addActionListener(new RestoreAction());
        
        panelRestore.add(fileFieldSource, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelRestore.add(buttonRestore,  Layout.getGBC( 1, 0, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 5, 5), 0, 0));        
    
        panelRestore.setToolTipText(DcResources.getText(Repository.tooltips.tpRestore));
        
        panelRestore.setBorder(
                ComponentFactory.getTitleBorder(DcResources.getText(Repository.labels.lblRestore)));
        
        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel(false);
        panelLog.setLayout(Layout.getGBL());
        panelLog.setBackground(color);
        
        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));
        
        panelLog.setBorder(ComponentFactory.getTitleBorder(
                           DcResources.getText(Repository.labels.lblLog)));        
        
        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel(false);
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.setBackground(color);
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));         
        
        
        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        
        JButton buttonClose = ComponentFactory.getButton(
                DcResources.getText(Repository.labels.lblClose));
        
        buttonClose.addActionListener(new CloseAction());
        
        this.getContentPane().add(panelBackup,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelRestore, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelLog,     Layout.getGBC( 0, 2, 1, 1, 10.0, 10.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelProgress,Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(buttonClose,  Layout.getGBC( 0, 4, 1, 1, 0.0, 0.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        pack();
        
        Dimension size = DcSettings.getValueAsDimension(Repository.settings.stBackupDialogSize);
        setSize(size);
        
        setCenteredLocation();
    }

    private class BackupAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            BackupThread thread = new BackupThread();
            thread.start();
        }
    }      

    private class RestoreAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            RestoreThread thread = new RestoreThread();
            thread.start();
        }
    }      
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }      
    
    /**
     * Image file filter
     */
    private class BackupFileFilter extends FileFilter {
        
        public boolean accept(File file) {
            if (file.isDirectory()) {
                return true;
            } else if (Utilities.getExtension(file).equals("bck")) {
                return true;
            } else {
                return false;
            }
        }
        
        public String getDescription() {
            return DcResources.getText(Repository.labels.lblBckFileFilter);
        }
    }   
    
    private class BackupThread extends Thread {
        public void run() {
            String target = fileFieldTarget.getFileAsString();
            
            textLog.setText("");
            if (target != null && target.length() > 0) {
                addMessage(DcResources.getText(Repository.messages.msgStartBackup));
                
                buttonBackup.setEnabled(false);
            
                if (!Database.isClosed()) {
                    addMessage(DcResources.getText(Repository.messages.msgClosingDb));
                    Database.close();
                }

                File dataDir = new File(DataCrow.baseDir + "data" + File.separator);
                Collection files = getFiles();
                try {
                    byte b[] = new byte[512];
                    ZipOutputStream zout = getZipOutputStream(target);
                    
                    if (zout != null) {
                        int counter = 1;
                        for (Iterator iter = files.iterator(); iter.hasNext(); counter++) {
                            String file = dataDir + File.separator + iter.next();

                            if (!file.endsWith(".log")) {
                                InputStream in = new FileInputStream(file);
                                ZipEntry e = new ZipEntry(file.replace(File.separatorChar, '/'));
                                zout.putNextEntry(e);
                                int len = 0;
                                while ((len = in.read(b)) != -1) {
                                    zout.write(b, 0, len);
                                }
                                
                                addMessage(DcResources.getText(Repository.messages.msgCreatingBackupOfFile, file));

                                zout.closeEntry();
                            }
                            
                            updateProgressBar(counter);
                        }
                        zout.close();
                        
                        addMessage(DcResources.getText(Repository.messages.msgWritingBackupFile));
                    }
                } catch (Exception exp) {
                    addMessage(DcResources.getText(Repository.messages.msgBackupError, exp.getMessage()));
                    Messages.add(exp);
                }
            
                addMessage(DcResources.getText(Repository.messages.msgRestartingDb));
                WorkFlow.restartDatabase();
                addMessage(DcResources.getText(Repository.messages.msgBackupFinished));
            } else {
                new MessageBox(DcResources.getText(Repository.messages.msgSelectOutputDir), MessageBox._WARNING);
            }  
            buttonBackup.setEnabled(true);
        }        
        
    }
    
    private class RestoreThread extends Thread {
        
        public void run() {
        
            String source = fileFieldSource.getFileAsString();
            
            textLog.setText("");
            if (source != null && source.length() > 0) {
                addMessage(DcResources.getText(Repository.messages.msgStartRestore));
                
                buttonRestore.setEnabled(false);
                
                if (!Database.isClosed()) {
                    addMessage(DcResources.getText(Repository.messages.msgClosingDb));
                    Database.close();
                }
    
                 try {
                    ZipFile zf = new ZipFile(source);
                    
                    Enumeration list = zf.entries();
                    initProgressBar(zf.size());
                    int counter = 1;
                    while (list.hasMoreElements()) {
                        ZipEntry ze = (ZipEntry)list.nextElement();
        
                        String filename = ze.getName();
                        filename = DataCrow.baseDir + File.separator + "data" + File.separator +
                                   filename.substring(filename.lastIndexOf("/") + 1, filename.length());
                        
                        InputStream istr = zf.getInputStream(ze);
                        BufferedInputStream bis = new BufferedInputStream(istr);
                        FileOutputStream fos = new FileOutputStream(filename);
                        
                        int sz = (int)ze.getSize();
                        final int N = 1024;
                        byte buf[] = new byte[N];
                        int ln = 0;
                        while (sz > 0 &&  // workaround for bug
                            (ln = bis.read(buf, 0, Math.min(N, sz))) != -1) {
                                fos.write(buf, 0, ln);
                                sz -= ln;
                        }
                        bis.close();
                        fos.flush();
                        fos.close();
                        
                        addMessage(DcResources.getText(Repository.messages.msgRestoringFile, filename));
                        updateProgressBar(counter++);
                    }
                    
                } catch (Exception exp) {
                    addMessage(DcResources.getText(Repository.messages.msgRestoreError, exp.getMessage()));
                    new MessageBox(DcResources.getText(Repository.messages.msgRestoreError, exp.getMessage()), MessageBox._WARNING);
                    Messages.add(exp);
                }
             
                restartApplication();
                
                addMessage(DcResources.getText(Repository.messages.msgRestoreFinished));
             
             } else {
                 new MessageBox(DcResources.getText(Repository.messages.msgSelectBackupFile), MessageBox._WARNING);
             }
             buttonRestore.setEnabled(true);
        }
    }    
}
