package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.renderers.CheckBoxTableCellRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.dc.settings.definitions.Definitions;

public class FieldDefinitionsDialog extends DcDialog {

    private DefinitionPanel panelDefinitionsParent = null;
    private DefinitionPanel panelDefinitionsChild = null;
    
    public FieldDefinitionsDialog() {
        super(DataCrow.mainFrame);

        buildDialog();

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
    
        setTitle(DcResources.getText(Repository.labels.lblFieldSettings));
        setModal(true);
    }    

    private void close() {
        DcSettings.setValue(Repository.settings.stFieldSettingsDialogSize, getSize());
        dispose();
    }
    
    private void save() {
        panelDefinitionsParent.save();
        if (panelDefinitionsChild != null) 
            panelDefinitionsChild.save();
    }
    
    private void buildDialog() {
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);

        getContentPane().setLayout(Layout.getGBL());
        setBackground(color);

        /***********************************************************************
         * ACTIONS PANEL
         **********************************************************************/
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);

        JButton buttonSave = ComponentFactory.getButton(DcResources
                .getText(Repository.labels.lblSaveChanges));
        JButton buttonClose = ComponentFactory.getButton(DcResources
                .getText(Repository.labels.lblClose));

        buttonSave.addActionListener(new SaveAction());
        buttonClose.addActionListener(new CloseAction());

        panelActions.add(buttonSave, Layout.getGBC(0, 0, 1, 4, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 5), 0, 0));
        panelActions.add(buttonClose, Layout.getGBC(1, 0, 1, 4, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));

        /***********************************************************************
         * MAIN PANEL
         **********************************************************************/

        DCModule module = DCModules.getModule(DcSettings.getValueAsInt(Repository.settings.stModule));
        panelDefinitionsParent = new DefinitionPanel(module);
        panelDefinitionsParent.setBorder(ComponentFactory.getTitleBorder(module.getObjectName()));
        
        getContentPane().add(panelDefinitionsParent, Layout.getGBC(0, 0, 1, 1, 10.0, 10.0,
                             GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        if (module.getChildModule() != null) {
            panelDefinitionsChild = new DefinitionPanel(module.getChildModule());
            getContentPane().add(panelDefinitionsChild, Layout.getGBC(0, 1, 1, 1, 10.0, 10.0,
                                 GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets(5, 5, 5, 5), 0, 0));
            panelDefinitionsChild.setBorder(ComponentFactory.getTitleBorder(module.getChildModule().getObjectName()));
        }

        getContentPane().add(panelActions, Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                             GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));

        pack();
        
        Dimension dim = DcSettings.getValueAsDimension(Repository.settings.stFieldSettingsDialogSize);
        setSize(dim);

        setCenteredLocation();
    }

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }

    protected class DefinitionPanel extends JPanel {
        
        private final DcTable table;
        private DCModule module;
        
        public DefinitionPanel(DCModule module) {
            this.module = module;
            table = ComponentFactory.getDCTable(false, false);
            buildPanel();
        }
        
        private void buildPanel() {
            Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
            
            setLayout(Layout.getGBL());
            setBackground(color);
    
            table.setColumnCount(6);
    
            TableColumn columnHidden = table.getColumnModel().getColumn(5);
            table.removeColumn(columnHidden);
            
            TableColumn columnField = table.getColumnModel().getColumn(0);
            JTextField textField = ComponentFactory.getTextFieldDisabled();
            columnField.setCellEditor(new DefaultCellEditor(textField));
            columnField.setHeaderValue(DcResources.getText(Repository.labels.lblOriginalLabel));
            
            TableColumn columnFieldName = table.getColumnModel().getColumn(1);
            DcShortTextField textName = ComponentFactory.getShortTextField(20);
            columnFieldName.setCellEditor(new DefaultCellEditor(textName));
            columnFieldName.setHeaderValue(DcResources.getText(Repository.labels.lblLabel));
    
            TableColumn columnEnabled = table.getColumnModel().getColumn(2);
            JCheckBox checkEnabled = new JCheckBox();
            checkEnabled.addActionListener(new DependencyCheckAction());
            columnEnabled.setCellEditor(new DefaultCellEditor(checkEnabled));
            columnEnabled.setCellRenderer(new CheckBoxTableCellRenderer());
            columnEnabled.setHeaderValue(DcResources.getText(Repository.labels.lblEnabledField));            
            
            TableColumn columnVisible = table.getColumnModel().getColumn(3);
            JCheckBox checkVisibility = new JCheckBox();
            columnVisible.setCellEditor(new DefaultCellEditor(checkVisibility));
            checkVisibility.addActionListener(new DependencyCheckAction());
            columnVisible.setCellRenderer(new CheckBoxTableCellRenderer());
            columnVisible.setHeaderValue(DcResources.getText(Repository.labels.lblVisible));
    
            TableColumn columnRequired = table.getColumnModel().getColumn(4);
            JCheckBox checkRequired = new JCheckBox();
            checkRequired.addActionListener(new DependencyCheckAction());
            columnRequired.setCellEditor(new DefaultCellEditor(checkRequired));
            columnRequired.setCellRenderer(new CheckBoxTableCellRenderer());
            columnRequired.setHeaderValue(DcResources.getText(Repository.labels.lblRequired));
    
            applyDefinitions();
    
            JScrollPane scroller = new JScrollPane(table);
            scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
    
            JLabel labelNav = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblMove));
            JButton buttonUp = ComponentFactory.getButton(IconLibrary._icoNavigateUp);
            JButton buttonDown = ComponentFactory.getButton(IconLibrary._icoNavigateDown);
    
            buttonUp.addActionListener(new RowUpAction());
            buttonDown.addActionListener(new RowDownAction());
    
            add(scroller, Layout.getGBC(0, 0, 1, 4, 50.0, 50.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                    new Insets(0, 0, 0, 0), 0, 0));
            add(labelNav, Layout.getGBC(1, 0, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(0, 5, 5, 5), 0, 0));
            add(buttonUp, Layout.getGBC(1, 1, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(0, 5, 0, 5), 0, 0));
            add(buttonDown, Layout.getGBC(1, 2, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(5, 5, 0, 5), 0, 0));
    
            table.applyHeaders();
        }
        
        private void checkDependencies() {
            table.cancelEdit();
            int row = table.getSelectedRow();
            
            if (row > -1) {
                DcFieldDefinition definition = getDefinition(row);
                
                if (!definition.isEnabled()) {
                    table.setValueAt(Boolean.FALSE, row, 5);
                    table.setValueAt(Boolean.FALSE, row, 4);
                }
                
                if (!definition.isVisible()) {
                    table.setValueAt(Boolean.FALSE, row, 4);
                }
            }
        }
        
        private void save() {
            DcSettings.setValue(module.getFieldDefinitionsSettingsKey(), getDefinitions());
            
            if (module.getSearchPanel() != null)
                module.getSearchPanel().applyDefinitions();
            
            if (module.getInsertPanel() != null)
                module.getInsertPanel().applyDefinitions();
        }    
        
        public void applyDefinitions() {
            Definitions definitions = DcSettings.getValueAsDefinitionGroup(module.getFieldDefinitionsSettingsKey());
            table.clearTable();
            for (Iterator iter = definitions.iterator(); iter.hasNext();) {
                DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                table.addRow(definition.getDisplayValues(module));
            }
        }        
        
        private DcFieldDefinition getDefinition(int row) {
            String name = (String) table.getValueAt(row, 1, true);
            boolean enabled = ((Boolean) table.getValueAt(row, 2, true)).booleanValue();
            boolean visible = ((Boolean) table.getValueAt(row, 3, true)).booleanValue();
            boolean required = ((Boolean) table.getValueAt(row, 4, true)).booleanValue();
            DcField field = (DcField) table.getValueAt(row, 5, true);
            
            return new DcFieldDefinition(field.getIndex(), name, enabled, required, visible);
        }
        
        public DcFieldDefinitions getDefinitions() {
            table.cancelEdit();
            DcFieldDefinitions definitions = new DcFieldDefinitions();
            for (int i = 0; i < table.getRowCount(); i++) {
                definitions.addDefinition(getDefinition(i));
            }
            return definitions;
        }  
        
        private class RowUpAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                table.moveRowUp();
            }
        }
        
        private class RowDownAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                table.moveRowDown();
            }
        }    
        
        private class DependencyCheckAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                checkDependencies();
            }
        }          
    }
}
