package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ItemTypeDialog extends DcDialog {

    private int selectedModule = DCModules._SOFTWARE;
    
    public ItemTypeDialog(JFrame parent) {
        super(parent);
        setTitle(DcResources.getText(Repository.labels.lblSelectModule));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        setModal(true);
        buildDialog();
    }
    
    public int getSelectedModule() {
        return selectedModule;
    }
    
    private void close() {
        DcSettings.setValue(Repository.settings.stModuleSelectDialogSize, getSize());
        dispose();
    }
    
    
    private void buildDialog() {
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        getContentPane().setLayout(Layout.getGBL());
        setBackground(color);
        
        //**********************************************************
        //Create Backup Panel
        //**********************************************************
        JPanel panelModules = new JPanel();
            
        panelModules.setLayout(Layout.getGBL());
        panelModules.setBackground(color);
        
        final ButtonGroup buttonGroup = new ButtonGroup();
        class SelectModuleAction implements ItemListener {
            public void itemStateChanged(ItemEvent ev) {
                String command = buttonGroup.getSelection().getActionCommand();
                selectedModule = Integer.parseInt(command);
            }
        }         
        
        DcLongTextField textHelp = ComponentFactory.getLongTextField(8000);
        textHelp.setBorder(null);
        JScrollPane scroller = new JScrollPane(textHelp);
        scroller.setBorder(null);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        textHelp.setEditable(false);
        textHelp.setBackground(color);        
        textHelp.setText(DcResources.getText(Repository.labels.lblSelectModuleHelp));
        panelModules.add(textHelp, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));        
        
        int y = 1;
        for (Iterator iter = DCModules.getModules(); iter.hasNext(); ) {
            DCModule module = (DCModule) iter.next();

            if (module.isTopModule() && module.getIndex() != DCModules._GENERIC) {
                JRadioButton radioButton = ComponentFactory.getRadioButton(module.getName(), module.getIcon(), "" + module.getIndex());
                radioButton.addItemListener(new SelectModuleAction());
                buttonGroup.add(radioButton);
                panelModules.add(radioButton, Layout.getGBC( 0, y++, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets( 0, 5, 5, 5), 0, 0));
            }
        }

        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelAction = new JPanel(false);
        panelAction.setLayout(Layout.getGBL());
        panelAction.setBackground(color);

        JButton buttonOk = ComponentFactory.getButton(
                DcResources.getText(Repository.labels.lblOK));
        
        buttonOk.addActionListener(new OkAction());
        
        panelAction.add(buttonOk, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                        new Insets(0, 5, 0, 0), 0, 0));         
        
        
        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        
        
        this.getContentPane().add(panelModules,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelAction, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 0, 5, 5, 0), 0, 0));

        pack();
        Dimension size = DcSettings.getValueAsDimension(Repository.settings.stModuleSelectDialogSize);
        setSize(size);
        setCenteredLocation();
    }

    
    
    private class OkAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }      
}
