/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.processes.http.AmazonBookUpdate;
import net.sf.dc.processes.http.AmazonSoftwareUpdate;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

/**
 * Searches Amazon.com for information about programs and games
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public abstract class AmazonSearchForm extends InternetSearchForm {
    
    private JComboBox comboServers;
    protected JTextField textSearch;
    protected JButton buttonSearch;
    
    public AmazonSearchForm(DcObject dco, ItemForm itemForm) {
        super(itemForm, dco, itemForm);
        setTitle(DcResources.getText(Repository.labels.lblAmazonUpdateProcess, module.getObjectName()));
    }
    
    public AmazonSearchForm(DcObject dco, int tab) {
        super(DataCrow.mainFrame, dco, tab);
        setTitle(DcResources.getText(Repository.labels.lblAmazonUpdateProcess, module.getObjectName()));
    }
    
    public void setDefaultValues() {
        setSize(DcSettings.getValueAsDimension(
                Repository.settings.stAmazonSearchFormViewSize));        
        
        if (dco != null) {
        	textSearch.setText(dco.getValue(Software._A_TITLE).toString());
        }
        
        setCenteredLocation();
    }
    
    public void setSearchString(String s) {
        textSearch.setText(s);
    }

    public void setCancelMode() {
        cancelled = false;
        buttonSearch.setEnabled(false);
        buttonStopSearch.setEnabled(true);
    }    
    
    public void setSearchMode() {
        buttonSearch.setEnabled(true);
        buttonStopSearch.setEnabled(false);        
    }
    
    public void saveSettings() {
        super.saveSettings();
        DcSettings.setValue(Repository.settings.stAmazonSearchFormViewSize, getSize());
        DcSettings.setValue(Repository.settings.stAmazonServer, comboServers.getSelectedItem().toString());  
    }    
    
    public void setVisible(boolean b) {
        try {
            String server = 
                DcSettings.getValueAsString(Repository.settings.stAmazonServer);
            comboServers.setSelectedItem(server);
        } catch (Exception exp) {}
        super.setVisible(b);
    }    
    
    public void startSearch() {
        setCancelMode();
        saveSettings();
        
        String keywords = textSearch.getText();
        if (keywords.length() >= 4) {
            if (module.getIndex() == DCModules._SOFTWARE) {
            	process = new AmazonSoftwareUpdate(this, textSearch.getText(), checkBroadband.isSelected());
            } else {
                process = new AmazonBookUpdate(this, textSearch.getText(),checkBroadband.isSelected());
            }
        	process.setPriority(Thread.MAX_PRIORITY);
        	process.start();
        } else {
            new MessageBox(DcResources.getText(Repository.messages.msgSearchStringLength, "4"),
                           MessageBox._WARNING);            
            setSearchMode();
        }
    }
    
    public JPanel getSearchPanel() {
        JPanel panel = new JPanel(false);
        panel.setLayout(Layout.getGBL());
        panel.setBackground(color);
        
        textSearch = ComponentFactory.getTextField();
        textSearch.setMinimumSize(new Dimension(100,19));
        
        buttonSearch =  ComponentFactory.getButton(DcResources.getText(Repository.labels.lblFind));
        buttonSearch.addActionListener(new SearchAction());
        buttonSearch.setMnemonic(KeyEvent.VK_F);
        
        JLabel labelServer = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblInternetServer));
        comboServers = ComponentFactory.getComboBox();
        comboServers.addItem("http://xml.amazon.com");
        comboServers.addItem("http://xml.amazon.co.uk");
        comboServers.addItem("http://xml.amazon.de");
        comboServers.addItem("http://xml.amazon.fr");
        comboServers.addItem("http://xml.amazon.ca");
        comboServers.addItem("http://xml.amazon.co.jp");

        buttonStopSearch =  ComponentFactory.getButton(DcResources.getText(Repository.labels.lblStop));
        buttonStopSearch.addActionListener(new StopSearchAction());        
        
        panel.setLayout(Layout.getGBL());          
        panel.add(labelServer,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                new Insets(5, 5, 5, 5), 0, 0));                         
        panel.add(comboServers, Layout.getGBC( 1, 0, 1, 1, 80.0, 80.0
                               ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                new Insets(5, 5, 5, 5), 0, 0));                           
        panel.add(textSearch,   Layout.getGBC( 0, 1, 2, 1, 90.0, 90.0
                               ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                new Insets(5, 5, 5, 5), 0, 0));                         
        panel.add(buttonSearch, Layout.getGBC( 2, 1, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonStopSearch,
                                Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                new Insets(5, 5, 5, 5), 0, 0));
        
        try {
            String server = DcSettings.getValueAsString(Repository.settings.stImdbServer);
            comboServers.setSelectedItem(server);
        } catch (Exception exp) {}        
        
        return panel;
    }
    
    private class SearchAction implements ActionListener{
        public void actionPerformed(ActionEvent e) {
            startSearch();   
        }
    }    
}
