/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards;

import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JLabel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

/**
 * @author Robert Jan van der Waals
 * @version 2.2
 * @since 1.4
 */
public class NameWizardPanel extends WizardPanel {
    
    DcShortTextField textField;
    
    public NameWizardPanel(DcObject dco) {
        this.dco = dco;
        build();
    }
    
    public DcObject apply() throws WizardException {
        String s = textField.getText().trim(); 
        if (s.length() == 0) { 
            throw new WizardException(DcResources.getText(Repository.messages.msgEmptyName));
        }
        
        dco.setValue(1, textField.getText());
        return dco;
    }
    
    public String getHelpText() {
        return DcResources.getText(Repository.messages.msgEnterName, dco.getModule().getObjectName());
    }
    
    private void build() {
        DcField field = dco.getField(1);
        
        textField = ComponentFactory.getShortTextField(field.getMaximumLength());
        JLabel labelName = ComponentFactory.getLabel(field.getLabel());

        setLayout(Layout.getGBL());
        add(labelName, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 5), 0, 0));
        add(textField, Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
    }
}