/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.AudioCDQuickViewPane;
import net.sf.dc.console.audiocd.ImportAudioCDDialog;
import net.sf.dc.console.audiocd.InsertAudioCDPanel;
import net.sf.dc.console.audiocd.SearchAudioCDPanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Audio CD module
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public class AudioCDModule extends DcMediaModule {
    
    private ImportAudioCDDialog importAudioCDDialog = null;
    
    
    /**
     * Initializes this module
     */
    public AudioCDModule() {
        super(DCModules._AUDIOCD,
              true,
              DcResources.getText(Repository.systemlabels.sysAudioCD),
              "audiocds",
              "aucd",
              " audiocds aucd LEFT JOIN audiotracks autr ON (aucd.ID = autr.Album)",
              false,
              false,
              true);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }
    
    public String getObjectName() {
    	return DcResources.getText(Repository.systemlabels.sysAudioCD);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
										        DcResources.getText(Repository.labels.lblAudioCDImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuImport);
        
    	return menu;
    }
    
    public DcObject getDCObject() {
        return new AudioCD();
    }

    protected void initializeUI() {
        insertPanel = new InsertAudioCDPanel();
        searchPanel = new SearchAudioCDPanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioCD;
    }

    
    public void startImport() {
        if (importAudioCDDialog == null) 
            importAudioCDDialog = new ImportAudioCDDialog();

        importAudioCDDialog.setVisible(true);
        importAudioCDDialog.startSearch();
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new AudioCDQuickViewPane();  
        
        return quickView;
    }
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
											  String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._B_DESCRIPTION, false, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._D_LOCATION , false, 1, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._E_RATING , true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._F_ARTIST, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._G_GENRE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._H_PICTUREFRONT, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._I_PICTUREBACK , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._J_PICTURECD , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._K_DISCID , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._U5_USER_NUMERIC2, false, 0, 1));        
        
        return definitions;
    }
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(AudioCD._F_ARTIST),
                    new DcField(AudioCD._F_ARTIST, getIndex(), "Artist",  
                                false, true, false, true, false, 
                                4000, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING,
                                "Artist"));
        fields.put( new Integer(AudioCD._G_GENRE),
                    new DcField(AudioCD._G_GENRE, getIndex(), "Genre",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Genre"));
        fields.put( new Integer(AudioCD._H_PICTUREFRONT),
                    new DcField(AudioCD._H_PICTUREFRONT, getIndex(), "Picture Front",  
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT,
                                "PictureFront"));
        fields.put( new Integer(AudioCD._I_PICTUREBACK),
                    new DcField(AudioCD._I_PICTUREBACK, getIndex(), "Picture Back",  
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT,
                                "PictureBack"));
        fields.put( new Integer(AudioCD._J_PICTURECD),
                    new DcField(AudioCD._J_PICTURECD, getIndex(), "Picture CD",  
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT,
                                "PictureCD"));
        fields.put( new Integer(AudioCD._K_DISCID),
                    new DcField(AudioCD._K_DISCID, getIndex(), "Disc ID",  
                                false, true, false, true, true, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "DiscID"));

    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }       
}
