/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JMenu;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.InsertAudioTrackPanel;
import net.sf.dc.console.audiocd.SearchAudioTrackPanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Audio Track module
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class AudioTrackModule extends DcMediaModule {
    
    /**
     * Initializes this module
     */
    public AudioTrackModule() {
        super(DCModules._AUDIOTRACK,
              false,
              DcResources.getText(Repository.systemlabels.sysAudioTrack),
              "audiotracks",
              "autr",
              "audiotracks",
              false,
              false,
              false);
    }

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysAudioTrack);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }
    
    public JMenu getToolsMenu() {
        return null;
    }
    
    /**
     * Return the object managed by this module 
     * @see AudioTrack
     */
    public DcObject getDCObject() {
        return new AudioTrack();
    }
    
    public void startImport() {
    }
    
    public QuickViewPanel getQuickView() {
        return null;
    }
    
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertAudioTrackPanel();
     	searchPanel = new SearchAudioTrackPanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioTrack;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        return new ReportFieldDefinitions();
    }    
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(AudioTrack._F_TRACKNUMBER), 
                    new DcField(AudioTrack._F_TRACKNUMBER, getIndex(), "Track", 
                                false, true, false, false, false,
                                255, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Track"));
        fields.put( new Integer(AudioTrack._G_LYRICS), 
                    new DcField(AudioTrack._G_LYRICS, getIndex(), "Lyric", 
                                false, true, false, true, false, 
                                255, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING,
                                "Lyric"));      
        fields.put( new Integer(AudioTrack._H_PLAYLENGTH), 
                    new DcField(AudioTrack._H_PLAYLENGTH, getIndex(), "Playlength", 
                                false, true, false, false, false,
                                255, ComponentFactory._TIMEFIELD, Repository.datatypes._INTEGER,
                                "PlayLength"));
        fields.put( new Integer(AudioTrack._I_ALBUM), 
                    new DcField(AudioTrack._I_ALBUM, getIndex(), "Album", 
                                false, true, true, false, false,
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._PARENTREFERENCE,
                                "Album"));    
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
}
