/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.book.BookCategoryForm;
import net.sf.dc.console.book.BookQuickViewPane;
import net.sf.dc.console.book.InsertBookPanel;
import net.sf.dc.console.book.SearchBookPanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Book module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class BookModule extends DcMediaModule {
    
    /**
     * Initializes this module
     */
    public BookModule() {
        super(DCModules._BOOK,
              true,
              DcResources.getText(Repository.systemlabels.sysBook),
              "books",
              "boo",
              "books",
              false,
              true,
              false);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoBookModule;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysBook);
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new BookQuickViewPane();  

        return quickView;        
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));

        JMenuItem menuCat = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuCategory16,
                                                DcResources.getText(Repository.systemlabels.sysBookCategoryPlural));
        menuCat.addActionListener(new CategoryFormMenuAction());
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                DcResources.getText(Repository.labels.lblSoftwareImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuCat);
        menu.addSeparator();
        menu.add(menuImport);
        
        return menu;
    }      
        
    /**
     * Return the object managed by this module 
     * @see Book
     */
    public DcObject getDCObject() {
        return new Book();
    }
        
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertBookPanel();
        searchPanel = new SearchBookPanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsBook;
    }
    
    public void startImport() {
        new MessageBox("Not implemented (yet) ! Sorry for the inconvenience. We suggest using the online search instead!", MessageBox._INFORMATION);
    }
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
							     			  String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(Book._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Book._B_DESCRIPTION, true, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(Book._F_PUBLISHER, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Book._G_AUTHOR, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Book._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._H_WEBPAGE, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._I_CATEGORY, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._J_ISBN, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._K_PICTUREFRONT, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._D_LOCATION, false, 1, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._E_RATING , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Book._U5_USER_NUMERIC2, false, 0, 1));        

        return definitions;
    }    
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        super.initializeFields();
        
        fields.put( new Integer(Book._F_PUBLISHER),
                    new DcField(Book._F_PUBLISHER, getIndex(), "Publisher",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Publisher"));
        fields.put( new Integer(Book._G_AUTHOR),
                    new DcField(Book._G_AUTHOR, getIndex(), "Author",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Author"));
        fields.put( new Integer(Book._H_WEBPAGE),
                    new DcField(Book._H_WEBPAGE, getIndex(), "Webpage",  
                                false, true, false, false, false, 
                                255, ComponentFactory._URLFIELD, Repository.datatypes._STRING,
                                "Webpage"));
        fields.put( new Integer(Book._I_CATEGORY),
                    new DcField(Book._I_CATEGORY, getIndex(), "Category",  
                                false, true, false, true, false, 
                                255, ComponentFactory._BOOKCATEGORYCOMBO, Repository.datatypes._TYPEREFERENCE,
                                "Type"));                             
        fields.put( new Integer(Book._J_ISBN),
                    new DcField(Book._J_ISBN, getIndex(), "ISBN",  
                                false, true, false, true, false, 
                                20, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "ISBN")); 
        fields.put( new Integer(Book._K_PICTUREFRONT),
                    new DcField(Book._K_PICTUREFRONT, getIndex(), "PictureFront",  
                                false, true, false, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT,
                                "PictureFront")); 
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class CategoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            BookCategoryModule bookCatMod = 
                (BookCategoryModule) DCModules.getModule(DCModules._BOOKCAT);
            BookCategoryForm categoryForm = bookCatMod.getForm(); 
            categoryForm.setVisible(true);
        }
    }        
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }      
}
