package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JMenu;

import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.generic.GenericQuickViewPane;
import net.sf.dc.console.generic.SearchGenericObjectPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

public class GenericModule extends DcMediaModule {
    
    /**
     * Initializes this module
     */
    public GenericModule() {
        super(DCModules._GENERIC,
              true,
              DcResources.getText(Repository.systemlabels.sysGenericModule),
              "",
              "",
              "",
              false,
              false,
              false);
    }

    public String getObjectName() {
        return "";
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoGenericModule;
    }
    
    public JMenu getToolsMenu() {
        return null;
    }
    
    /**
     * Return the object managed by this module 
     * @see AudioTrack
     */
    public DcObject getDCObject() {
        return new GenericObject();
    }
    
    public void startImport() {
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new GenericQuickViewPane();  

        return quickView;   
    }
    
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = null;
        searchPanel = new SearchGenericObjectPanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsGeneric;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._B_DESCRIPTION, true, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._D_LOCATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._E_RATING, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._SYS_MODULE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(GenericObject._U5_USER_NUMERIC2, false, 0, 1));
        return definitions;
    }    
    
    protected void initializeFields() {
        super.initializeFields();
        fields.put(new Integer(DcObject._SYS_MODULE), getField(DcObject._SYS_MODULE));          
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }    
}
