/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.movie.InsertMoviePanel;
import net.sf.dc.console.movie.MovieQuickViewPane;
import net.sf.dc.console.movie.SearchMoviePanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMovieCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Movie module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class MovieModule extends DcMediaModule {
    
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame, 
                           DcResources.getText(Repository.labels.lblMovieFileImport));
    
    /**
     * Initializes this module
     */
    public MovieModule() {
        super(DCModules._MOVIE,
              true,
              DcResources.getText(Repository.systemlabels.sysMovie),
              "movies",
              "mov",
              "movies",
              true,
              false,
              false);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoMovieModule;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMovie);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                DcResources.getText(Repository.labels.lblMovieFileImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuImport);
        
        return menu;
    }
    
    /**
     * Return the object managed by this module 
     * @see Movie
     */
    public DcObject getDCObject() {
        return new Movie();
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new MovieQuickViewPane();  

        return quickView;            
    }
    
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertMoviePanel();
        searchPanel = new SearchMoviePanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMovie;
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._MOVIE));
        importCDDialog.setVisible(true);
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmDirs indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
							     			  String nameOfCD) {

        return new ImportMovieCD(ui,
                                 sourceDir,
                                 confirmDirs,
                                 recursiveDirs,
                                 nameOfCD);
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(Movie._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Movie._F_TITLE_LOCAL, false, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Movie._B_DESCRIPTION, true, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(Movie._G_WEBPAGE, false, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Movie._H_GENRE, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._I_ACTORS, false, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(Movie._J_DIRECTOR, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._K_COUNTRY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._L_PLAYLENGTH, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._M_FILENAME, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._N_VIDEOCODEC, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._O_AUDIOCODEC, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._P_WIDTH, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._Q_HEIGHT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._R_FPS, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._S_FRAMES, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._T_AUDIOBITRATE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U_AUDIOSAMPLINGRATE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._V_AUDIOCHANNEL, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._W_VIDEOBITRATE , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._X_PICTUREFRONT , true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._Y_PICTUREBACK , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._Z_PICTURECD , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._0_LANGUAGE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._1_AUDIOLANGUAGE , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._2_SUBTITLELANGUAGE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._3_CERTIFICATION , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._4_FILESIZE , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._D_LOCATION , false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._E_RATING , true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U5_USER_NUMERIC2, false, 0, 1));        
        
        return definitions;
    }      
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(Movie._F_TITLE_LOCAL),
                    new DcField(Movie._F_TITLE_LOCAL, getIndex(), "Title (local)",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Title2"));
        fields.put( new Integer(Movie._G_WEBPAGE),
                    new DcField(Movie._G_WEBPAGE, getIndex(), "Webpage",  
                                false, true, false, false, false, 
                                255, ComponentFactory._URLFIELD, Repository.datatypes._STRING,
                                "Webpage"));
        fields.put( new Integer(Movie._H_GENRE),
                    new DcField(Movie._H_GENRE, getIndex(), "Genre",  
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Genre"));
        fields.put( new Integer(Movie._I_ACTORS),
                    new DcField(Movie._I_ACTORS, getIndex(), "Actors",  
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING,
                                "Actors"));
        fields.put( new Integer(Movie._J_DIRECTOR),
                    new DcField(Movie._J_DIRECTOR, getIndex(), "Director",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Director"));
        fields.put( new Integer(Movie._K_COUNTRY),
                    new DcField(Movie._K_COUNTRY, getIndex(), "Country",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Country"));    

        fields.put( new Integer(Movie._L_PLAYLENGTH),
                    new DcField(Movie._L_PLAYLENGTH, getIndex(), "Playlength",  
                                false, true, false, false, false,
                                10, ComponentFactory._TIMEFIELD, Repository.datatypes._INTEGER,
                                "PlayLength")); 
        fields.put( new Integer(Movie._M_FILENAME),
                    new DcField(Movie._M_FILENAME, getIndex(), "Filename",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Filename"));
        fields.put( new Integer(Movie._N_VIDEOCODEC),
                    new DcField(Movie._N_VIDEOCODEC, getIndex(), "Video Codec",  
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "VideoCodec"));
        fields.put( new Integer(Movie._O_AUDIOCODEC),
                    new DcField(Movie._O_AUDIOCODEC, getIndex(), "Audio Codec",  
                                false, true, false, true, true, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "AudioCodec"));
        fields.put( new Integer(Movie._P_WIDTH),
                    new DcField(Movie._P_WIDTH, getIndex(), "Width",  
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Width"));                                         
        fields.put( new Integer(Movie._Q_HEIGHT),
                    new DcField(Movie._Q_HEIGHT, getIndex(), "Height",  
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Height"));                                         
        fields.put( new Integer(Movie._R_FPS),
                    new DcField(Movie._R_FPS, getIndex(), "Fps",  
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Fps"));
        fields.put( new Integer(Movie._S_FRAMES),
                    new DcField(Movie._S_FRAMES, getIndex(), "Frames",  
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Frames"));
        fields.put( new Integer(Movie._T_AUDIOBITRATE),
                    new DcField(Movie._T_AUDIOBITRATE, getIndex(), "Audio Bitrate",  
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "AudioBitRate"));
        fields.put( new Integer(Movie._U_AUDIOSAMPLINGRATE),
                    new DcField(Movie._U_AUDIOSAMPLINGRATE, getIndex(), "Audio Sample Rate",  
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "AudioSampleRate"));
        fields.put( new Integer(Movie._V_AUDIOCHANNEL),
                    new DcField(Movie._V_AUDIOCHANNEL, getIndex(), "Audio Channels", 
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "AudioChannels"));
        fields.put( new Integer(Movie._W_VIDEOBITRATE),
                    new DcField(Movie._W_VIDEOBITRATE, getIndex(), "Video Bitrate", 
                                false, true, false, true, true, 
                                10, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "VideoBitRate"));
        fields.put( new Integer(Movie._X_PICTUREFRONT),
                    new DcField(Movie._X_PICTUREFRONT, getIndex(), "Picture Front", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureFront"));                                         
        fields.put( new Integer(Movie._Y_PICTUREBACK),
                    new DcField(Movie._Y_PICTUREBACK, getIndex(), "Picture Back", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureBack"));
        fields.put( new Integer(Movie._Z_PICTURECD),
                    new DcField(Movie._Z_PICTURECD, getIndex(), "Picture CD", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureCD"));
        fields.put( new Integer(Movie._Z_PICTURECD),
                    new DcField(Movie._Z_PICTURECD, getIndex(), "Picture CD", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureCD"));
        fields.put( new Integer(Movie._0_LANGUAGE),
                    new DcField(Movie._0_LANGUAGE, getIndex(), "Language", 
                                false, true, false, true, false, 
                                30, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Language"));
        fields.put( new Integer(Movie._1_AUDIOLANGUAGE),
                    new DcField(Movie._1_AUDIOLANGUAGE, getIndex(), "Audio Language", 
                                false, true, false, true, false, 
                                50, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "AudioLanguage"));
        fields.put( new Integer(Movie._2_SUBTITLELANGUAGE),
                    new DcField(Movie._2_SUBTITLELANGUAGE, getIndex(), "Subtitle Language", 
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "SubtitleLanguage"));
        
        fields.put( new Integer(Movie._3_CERTIFICATION),
                    new DcField(Movie._3_CERTIFICATION, getIndex(), "Certification", 
                                false, true, false, true, true, 
                                255, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING, 
                                "Certification"));
        fields.put( new Integer(Movie._4_FILESIZE),
                    new DcField(Movie._4_FILESIZE, getIndex(), "File Size", 
                                false, true, false, true, true, 
                                15, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Filesize"));              
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }      
}
