/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.musicfile.InsertMusicFilePanel;
import net.sf.dc.console.musicfile.MusicFileQuickViewPane;
import net.sf.dc.console.musicfile.SearchMusicFilePanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMusicFilesCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Music module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class MusicFileModule extends DcMediaModule {
    
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame, DcResources.getText(Repository.labels.lblMusicFileImport));
    
    /**
     * Initializes this module
     */
    public MusicFileModule() {
        super(DCModules._MUSIC,
              true,
              DcResources.getText(Repository.systemlabels.sysMusicFile),
              "musicfiles",
              "mufi",
              "musicfiles",
              false,
              false,
              false);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMusicFile);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));
        
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                DcResources.getText(Repository.labels.lblMusicFileImport));
        menuImport.addActionListener(new ImportCDAction());

        menu.add(menuImport);
        
        return menu;
    }    
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new MusicFileQuickViewPane();  

        return quickView;           
    }
    
    /**
     * Return the object managed by this module 
     * @see MusicFile
     */    
    public DcObject getDCObject() {
        return new MusicFile();
    }
        
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertMusicFilePanel();
        searchPanel = new SearchMusicFilePanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMusic;
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._MUSIC));
        importCDDialog.setVisible(true);
    }
    
    /**
     * Returns the music CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return
     */     
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
											  String nameOfCD) {
        return new ImportMusicFilesCD(ui, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._ID, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._A_TITLE, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._B_DESCRIPTION, true, 0, 3));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._F_ARTIST, true, 0, 2));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._G_ALBUM, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._C_YEAR, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._H_GENRE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._I_FILENAME, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._J_PLAYLENGTH, true, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._K_QUALITY, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._L_ENCODING, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._M_LYRIC, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._N_FILESIZE, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._D_LOCATION, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._E_RATING, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._U1_USER_LONGTEXT, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._U2_USER_SHORTTEXT1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._U3_USER_SHORTTEXT2, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._U4_USER_NUMERIC1, false, 0, 1));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._U5_USER_NUMERIC2, false, 0, 1));        
        
        return definitions;
    }       
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        super.initializeFields();
        
        fields.put( new Integer(MusicFile._F_ARTIST),
                    new DcField(MusicFile._F_ARTIST, getIndex(), "Artist",  
                                false, true, false, true, false, 
                                255, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING, 
                                "Artist"));
        fields.put( new Integer(MusicFile._G_ALBUM),
                    new DcField(MusicFile._G_ALBUM, getIndex(), "Album",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Album"));                                     
        fields.put( new Integer(MusicFile._H_GENRE),
                    new DcField(MusicFile._H_GENRE, getIndex(), "Genre",  
                                false, true, false, true, false, 
                                255, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING, 
                                "Genre"));
        fields.put( new Integer(MusicFile._I_FILENAME),
                    new DcField(MusicFile._I_FILENAME, getIndex(), "Filename",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Filename"));
        fields.put( new Integer(MusicFile._J_PLAYLENGTH),
                    new DcField(MusicFile._J_PLAYLENGTH, getIndex(), "Playlength",  
                                false, true, false, false, false, 
                                20, ComponentFactory._TIMEFIELD, Repository.datatypes._STRING, 
                                "Length"));
        fields.put( new Integer(MusicFile._K_QUALITY),
                    new DcField(MusicFile._K_QUALITY, getIndex(), "Quality",  
                                false, true, false, true, true, 
                                4, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Quality"));
        fields.put( new Integer(MusicFile._L_ENCODING),
                    new DcField(MusicFile._L_ENCODING, getIndex(), "Encoding",  
                                false, true, false, true, true, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Encoding"));
        fields.put( new Integer(MusicFile._M_LYRIC),
                    new DcField(MusicFile._M_LYRIC, getIndex(), "Lyric",  
                                false, true, false, true, false, 
                                8000, ComponentFactory._LONGTEXTFIELD, Repository.datatypes._STRING, 
                                "Lyric"));
        fields.put( new Integer(MusicFile._N_FILESIZE),
                    new DcField(MusicFile._N_FILESIZE, getIndex(), "File Size",  
                                false, true, false, true, true, 
                                15, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Filesize"));           
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }    
}
 