/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JMenu;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.util.Utilities;

/**
 * The Software category module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class SoftwareCategoryModule extends DCModule {
    
    private SoftwareCategoryForm form;
    
    /**
     * Initializes this module
     */
    public SoftwareCategoryModule() {
        super(DCModules._SOFTWARECAT,
              false,
              DcResources.getText(Repository.systemlabels.sysSoftwareCategory),
              "softwarecategories",
              "soca",
              "softwarecategories",
              false,
              false,
              false);
    }

    public ImageIcon getIcon() {
        return null;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysSoftwareCategory);
    }
    
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        form = new SoftwareCategoryForm(DataCrow.mainFrame);
        insertPanel = form.insertCat;
        searchPanel = form.searchCat;
    }

    public void startImport() {
    }
    
    public QuickViewPanel getQuickView() {
        return null;
    }
    
    public void setFieldOrder(int[] fields) {}       
    
    /**
     * Returns the form which holds the UI items for this Module.
     * In this, the Software Category module is an odd one. None
     * of the other modules have a specialized form.
     * 
     * The form consists of the insert and search panels
     */
    public SoftwareCategoryForm getForm() {
        return form;    
    }

    public JMenu getToolsMenu() {
        return null;
    }  

    /**
     * Return the object managed by this module 
     * @see SoftwareCategory
     */ 
    public DcObject getDCObject() {
        return new SoftwareCategory();
    }
    
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsSoftwareCategory;
    }
    
    /**
     * Returns null (no CD import process present for Software
     * Categories) 
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return null
     */ 
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return null;
    }  
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        return new ReportFieldDefinitions();
    }    
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        super.initializeFields();
        
        fields.put( new Integer(SoftwareCategory._A_NAME), 
                    new DcField(SoftwareCategory._A_NAME, getIndex(), "Name", 
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING,
                                "Name"));
        fields.put( new Integer(SoftwareCategory._B_ICON), 
                    new DcField(SoftwareCategory._B_ICON, getIndex(), "Icon", 
                                false, true, true, false, false,
                                255, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT,
                                "Icon"));
    }   
    
    public DcObject[] getDefaultData() throws Exception {
        String link = "file://" + Utilities.getCurrentDirectory() + File.separator + 
                      "icons_user" + File.separator;
            
        String[][] values = {{"3D Shooter",  link + "BlackOps.gif"},
                             {"Action Game", link + "BangBoy.gif"},
                             {"Adventure Game", link + "PlumpNdapper.gif"},
                             {"System Tool", link + "ZerToolz.gif"},
                             {"Emulator", link + "Zipdisk.gif"},
                             {"Internet", link + "NetObserver.gif"},
                             {"Racing game", link + "ToasterBot.gif"},
                             {"Strategy game", link + "TheCrownSitsHeavilyToday.gif"},
                             {"Converter", link + "GrrZap.gif"},
                             {"Role Playing Game", link + "He-MansAxe.gif"},
                             {"Family Game", link + "LeSmileyDeClassique.gif"},
                             {"Graphics Editor", link + "PhotoNetMagicHooya!.gif"},
                             {"Operating System", link + "BetterVirtualPC.gif"},
                             {"Editor", link + "Milton.gif"},
                             {"Driver", link + "UmaxAstra1200sscanner.gif"},
                             {"e-Books", link + "HitachiRasterOpsmonitor.gif"}};

        DcObject[] objects = new DcObject[values.length];
        for (int i = 0; i < values.length; i++) {
            DcObject dco = new SoftwareCategory();

            URL url = new URL(values[i][1]);
            String s = Utilities.fileToBase64String(url);

            dco.setValue(SoftwareCategory._A_NAME, values[i][0]);
            dco.setValue(SoftwareCategory._B_ICON, s);                            
            
            objects[i] = dco;
        }
        return objects;
    }
}
