/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.software.InsertSoftwarePanel;
import net.sf.dc.console.software.SearchSoftwarePanel;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.console.software.SoftwareQuickViewPane;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportSoftwareCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.util.Utilities;

/**
 * The Software module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.0
 */
public class SoftwareModule extends DcMediaModule {
    
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame,
                           DcResources.getText(Repository.labels.lblSoftwareImport));
    
    /**
     * Initializes this module
     */    
    public SoftwareModule() {
        super(DCModules._SOFTWARE,
              true,
              DcResources.getText(Repository.systemlabels.sysSoftware),
              "software",
              "soft",
              "software",
              false,
              true,
              false);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoSoftwareModule;
    }    
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysSoftware);
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new SoftwareQuickViewPane();  

        return quickView;    
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._SOFTWARE));
        importCDDialog.setVisible(true);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));

        JMenuItem menuCat = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuCategory16,
                                                DcResources.getText(Repository.systemlabels.sysSoftwareCategoryPlural));
        menuCat.addActionListener(new CatagoryFormMenuAction());

        menu.add(menuCat);
        menu.addSeparator();
        
        return menu;
    }      
    
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertSoftwarePanel();
        searchPanel = new SearchSoftwarePanel();    
    }
    
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsSoftware;
    }
    
    /**
     * Return the object managed by this module 
     * @see Software
     */    
    public DcObject getDCObject() {
        return new Software();
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return new ImportSoftwareCD(ui, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(Software._ID, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._A_TITLE, true, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._B_DESCRIPTION, true, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._F_DEVELOPER, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._G_PUBLISHER, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._H_PLATFORM, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._C_YEAR, true, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._I_WEBPAGE, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._J_FILENAME, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._K_TYPE, true, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._L_MULTI, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._M_PICTUREFRONT, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._N_PICTUREBACK, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._O_PICTURECD, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._P_SCREENSHOTONE, true, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._Q_SCREENSHOTTWO, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._R_SCREENSHOTTHREE, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._S_SERIALKEY, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._T_FILESIZE, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._D_LOCATION, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._E_RATING, true, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._U1_USER_LONGTEXT, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._U2_USER_SHORTTEXT1, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._U3_USER_SHORTTEXT2, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._U4_USER_NUMERIC1, false, 0, 100));
        definitions.addDefinition(new ReportFieldDefinition(Software._U5_USER_NUMERIC2, false, 0, 100));
        return definitions;
    }     
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(Software._F_DEVELOPER), 
                    new DcField(Software._F_DEVELOPER, getIndex(), "Developer", 
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Developer"));  
        fields.put( new Integer(Software._G_PUBLISHER), 
                    new DcField(Software._G_PUBLISHER, getIndex(), "Publisher", 
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Publisher"));  
        fields.put( new Integer(Software._H_PLATFORM), 
                    new DcField(Software._H_PLATFORM, getIndex(), "Platform", 
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Platform"));       
        fields.put( new Integer(Software._I_WEBPAGE), 
                    new DcField(Software._I_WEBPAGE, getIndex(), "Webpage", 
                                false, true, false, false, false, 
                                255, ComponentFactory._URLFIELD, Repository.datatypes._STRING, 
                                "Webpage"));      
        fields.put( new Integer(Software._J_FILENAME), 
                    new DcField(Software._J_FILENAME, getIndex(), "Filename", 
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "Filename"));        
        fields.put( new Integer(Software._K_TYPE), 
                    new DcField(Software._K_TYPE, getIndex(), "Type", 
                                false, true, false, true, false, 
                                255, ComponentFactory._SOFTWARECATEGORYCOMBO, Repository.datatypes._TYPEREFERENCE,
                                "Type"));       
        fields.put( new Integer(Software._L_MULTI), 
                    new DcField(Software._L_MULTI, getIndex(), "Multiplayer ?", 
                                false, true, false, true, false, 
                                4, ComponentFactory._YESNOCOMBO, Repository.datatypes._STRING, 
                                "Multiplayer"));
        fields.put( new Integer(Software._M_PICTUREFRONT), 
                    new DcField(Software._M_PICTUREFRONT, getIndex(), "Picture Front", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureFront"));
        fields.put( new Integer(Software._N_PICTUREBACK), 
                    new DcField(Software._N_PICTUREBACK, getIndex(), "Picture Back", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureBack"));
        fields.put( new Integer(Software._O_PICTURECD), 
                    new DcField(Software._O_PICTURECD, getIndex(), "Picture CD", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "PictureCD"));      
        fields.put( new Integer(Software._P_SCREENSHOTONE), 
                    new DcField(Software._P_SCREENSHOTONE, getIndex(), "Screenshot One", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "ScreenshotOne"));     
        fields.put( new Integer(Software._Q_SCREENSHOTTWO), 
                    new DcField(Software._Q_SCREENSHOTTWO, getIndex(), "Screenshot Two", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "ScreenshotTwo")); 
        fields.put( new Integer(Software._R_SCREENSHOTTHREE), 
                    new DcField(Software._R_SCREENSHOTTHREE, getIndex(), "Screenshot Three", 
                                false, true, true, false, false, 
                                4000, ComponentFactory._PICTUREFIELD, Repository.datatypes._OBJECT, 
                                "ScreenshotThree")); 
        fields.put( new Integer(Software._S_SERIALKEY), 
                    new DcField(Software._S_SERIALKEY, getIndex(), "Serial Key", 
                                false, true, false, false, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, Repository.datatypes._STRING, 
                                "SerialKey"));     
        fields.put( new Integer(Software._T_FILESIZE), 
                    new DcField(Software._T_FILESIZE, getIndex(), "Filesize", 
                            false, true, false, true, true, 
                                255, ComponentFactory._NUMBERFIELD, Repository.datatypes._INTEGER, 
                                "Filesize"));         
    }
    
    public DcObject[] getDefaultData() throws Exception {
        DcObject[] objects = new DcObject[1];
        
        Software software = new Software();
        software.markAsUnchanged();
        String link = "file://" + DataCrow.baseDir + "icons_system/about.jpg";

        software.setValue(Software._A_TITLE, "Data Crow 2.3");
        software.setValue(Software._B_DESCRIPTION, "There are many programs that claim to have a nice database to put your music in, and there are other programs that allow you to store your audio CD information, but Data Crow has it all under one roof!\n\n" +
                                                   "Data Crow helps you organize your software, music, book, and movie collections by using online databases from Amazon, IMDb, and freedb, as well as the tag information stored within your files.\n\n" +
                                                   "We think you will find Data Crow easy to use and a great all-rounder!");
        software.setValue(Software._E_RATING, new Integer(4));
        software.setValue(Software._H_PLATFORM, "all platforms");
        software.setValue(Software._C_YEAR, new Integer(2006));
        software.setValue(Software._I_WEBPAGE, "http://www.datacrow.net");
        software.setValue(Software._J_FILENAME, "/downloads/datacrow.zip");
        software.setValue(Software._S_SERIALKEY, "xxxx-xxxx-xxxx-xxxx");
        software.setValue(Software._D_LOCATION, "CD232");
        software.setValue(Software._T_FILESIZE, new Integer(4500));
        software.setValue(Software._U1_USER_LONGTEXT, "");
        software.setValue(Software._U2_USER_SHORTTEXT1, "");
        software.setValue(Software._U3_USER_SHORTTEXT2, "");
        software.setValue(Software._U4_USER_NUMERIC1, new Integer(0));
        software.setValue(Software._U5_USER_NUMERIC2, new Integer(0));
        
        try {
            software.setValue(Software._M_PICTUREFRONT, 
                                        Utilities.fileToBase64String(new URL(link)));
        } catch (Exception ignore) {} // picture error.. skip picture

        objects[0] = software;
        
        return objects;
    }
    
    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            SoftwareCategoryModule softCatMod = 
                (SoftwareCategoryModule) DCModules.getModule(DCModules._SOFTWARECAT);
            SoftwareCategoryForm categoryForm = softCatMod.getForm(); 
            categoryForm.setVisible(true);
        }
    }
}


