/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.cache;

import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JComboBox;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.book.BookCategory;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.messages.Messages;

/**
 * Manages the different Cache instances
 * @see net.sf.dc.core.objects.cache.Cache
 * 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.0
 */
public final class CacheManager {
    
    public static final int _SOFTWARECATEGORY = 0;
    public static final int _BOOKCATEGORY = 1;

    // container for Cache objects
    private static final LinkedList cache = new LinkedList();
    
    static {
        init();
    }

    /**
     * Initializes all Cache objects. For each of the Caches, all 
     * objects are loaded and the Caches are registered to this 
     * Cache Manager
     */
    private static void init() {
        try {
            SoftwareCategory softwareCat = new SoftwareCategory();
            Cache cSoftCat = new Cache(softwareCat);
            cache.add(cSoftCat);
            
            BookCategory bookCat = new BookCategory();
            Cache cBookCat = new Cache(bookCat);
            cache.add(cBookCat);
            
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }

    /**
     * Refreshes the cache for the specified cache index
     * @param index cache index
     */
    public static void refresh(int index) {
        getCache(index).load();
    }

    /**
     * Retrieves the cache for the given index
     * @param index cache index
     */
    private static Cache getCache(int index) {
        return (Cache) cache.get(index);
    }

    /**
     * Retrieves all Cached objects of the Cache with the given
     * index
     * @param index cache index
     */
    public static DcObject[] getObjects(int index) {
        return getCache(index).getCachedObjects();   
    }
    
    /**
     * Retrieves the cached object with the given display value
     * (toString() value)
     * @param s display value to search on
     */
    public static DcObject getObjectForDisplayValue(String s) {
        for (Iterator iter = cache.iterator(); iter.hasNext(); ) {
            Cache cache = (Cache) iter.next();
            DcObject o = cache.getObjectForDisplayValue(s); 
            if (o != null) {
                return o;
            }
        }
        return null;
    }

    /**
     * Retrieves the cached object with the given ID
     * @param id ID to search on
     */    
    public static DcObject getObjectForID(String id) {
        for (Iterator iter = cache.iterator(); iter.hasNext(); ) {
            Cache cache = (Cache) iter.next();
            DcObject o = cache.getObjectForID(id); 
            if (o != null) {
                return o;
            }
        }
        return null;
    }
    
    /**
     * Register a combo box. This combo box will be updated with
     * the currently cached values. Everytime the cache is refreshed, 
     * this combo box will be refreshed as well.
     * @param combo combo box to be registered
     * @param index cache index
     */
    public static void registerUIComponent(JComboBox combo, int index) {
        getCache(index).registerUIComponent(combo);
    }
}
