/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.helpsystem;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.help.HelpSet;
import javax.help.JHelp;
import javax.swing.JFrame;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Utilities;

/**
 * JavaHelp implementation. 
 * The help system is based on html pages with their own unique ID.
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.6
 */
public class JavaHelp {

    private static String helpIndex = "dc.Introduction";
    private JFrame helpFrame = new JFrame();
    
    /**
     * Sets the current help id. based on this ID the correct html
     * page is displayed
     * @param helpIndex
     */
    public static void setHelpIndex(String helpIndex) {
    	JavaHelp.helpIndex = helpIndex;
    }
    
    /**
     * Initializes the Java Help.
     * The helpfiles are retrieved using the Classloader
     */
	public JavaHelp() {
		JHelp helpViewer = null;
		try {
			// Get the classloader of this class.
			ClassLoader cl = JavaHelp.class.getClassLoader();
			// Use the findHelpSet method of HelpSet to create a URL referencing the helpset file.
			// Note that in this example the location of the helpset is implied as being in the same
			// directory as the program by specifying "jhelpset.hs" without any directory prefix,
			// this should be adjusted to suit the implementation.
            URL url = ClassLoader.getSystemResource("jHelpSet.hs");

            // Create a new JHelp object with a new HelpSet.
			helpViewer = new JHelp(new HelpSet(cl, url));
            helpViewer.setFont(new Font("Arial", Font.PLAIN, 11));
			// Set the initial entry point in the table of contents.
			helpViewer.setCurrentID(helpIndex);
		} catch (Exception e) {
            new MessageBox(DcResources.getText(Repository.messages.msgErrorOpeningHelp), MessageBox._ERROR);
            return;
		}

		// Create a new frame.
		helpFrame.setSize(DcSettings.getValueAsDimension(Repository.settings.stHelpFormSize));
        helpFrame.getContentPane().add(helpViewer);
        helpFrame.setTitle(DcResources.getText(Repository.labels.lblDataCrowHelp));
        helpFrame.setIconImage(IconLibrary._icoMenuHelp16.getImage());

        // Set a default close operation.
        helpFrame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                finish();
            }
        });
        
        helpFrame.pack();
        Dimension dim = Utilities.getCenteredWindowLocation(helpFrame.getSize());
        helpFrame.setLocation(dim.width, dim.height);
		helpFrame.setVisible(true);
	}
    
    private void finish() {
        DcSettings.setValue(Repository.settings.stHelpFormSize, helpFrame.getSize());
        helpFrame.dispose();
        helpFrame = null;
    }    
}