/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.messages.Messages;

/**
 * Imports data from a CD (or harddisk) location into the insert table
 * of the owner module
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 0.5
 */
public class ImportSoftwareCD extends ImportCDProcess {

    /**
     * Initializes (but does  not run) the Movie CD import process
     * @param ui used for displaying the progress
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmDirs indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportSoftwareCD(ImportCDDialog ui,
                            String sourceDir,
                            boolean confirmDirs,
                            boolean recursiveDirs,
                            String cdName) {
        super(ui, sourceDir, confirmDirs, recursiveDirs, cdName);
    }
   
    public String[] getExtensions() {
        return new String[] {};
    }
    
    protected int getModule() {
        return DCModules._SOFTWARE;
    }
  
    /**
     * Parses the source file for information. 
     */ 
    protected DcObject parseInformation(String filename) {
        Software software = new Software();
        try {
            software.setValue(Software._A_TITLE, getName(filename));
            software.setValue(Software._J_FILENAME, filename);
            software.setValue(Software._D_LOCATION, cdName);
            software.setValue(Software._T_FILESIZE, new Integer(getFileSize(filename)));
            return software;
        } catch (Exception exp) {
            Messages.add(exp);
            return null;
        }        
    }
}
