/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.io.File;

import javax.swing.JFrame;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcColorSelector;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.objects.book.BookCategory;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.dc.settings.definitions.Definitions;
import net.sf.dc.settings.definitions.ProgramDefinitions;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.jflexsett.settings.ui.SettingsView;

/**
 * Groups are created here, as well as the settings. A similar class like this 
 * needs to be implemented in the project which uses jFlexSett.
 * 
 * All settings and their values (and the settings groups) can
 * be found by referencing to the Settings class. 
 * 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 0.5
 */
public class DcSettings {

    public static final String _General = 
                                DcResources.getText(Repository.labels.lblGroupGeneral);
    public static final String _FileHandlers = 
        						DcResources.getText(Repository.labels.lblProgramDefinitions);
    public static final String _BackGroundColor = 
                                DcResources.getText(Repository.labels.lblColor);
    public static final String _TableColors =
                                DcResources.getText(Repository.labels.lblTableColorSettings);
    public static final String _OddRowColor = 
                                DcResources.getText(Repository.labels.lblOddRowColor);
    public static final String _EvenRowColor = 
                                DcResources.getText(Repository.labels.lblEvenRowColor);
    public static final String _HeaderColor =  
                                DcResources.getText(Repository.labels.lblTableHeaderColor);
    public static final String _RowSelectionColor =  
                                DcResources.getText(Repository.labels.lblRowSelectionColor);
    public static final String _HTTP =
                                DcResources.getText(Repository.labels.lblHTTPSettings);
    public static final String _Font =
                                DcResources.getText(Repository.labels.lblFont);
    
    private static Settings settings = new Settings();
    
    
    /**
     * Starts the initialization of the Settings and starts the UI
     */
    public DcSettings() {
        createGroups();
        createSettings();
        settings.setSettingsFile(new File(DataCrow.baseDir + "data/data_crow.properties"));
        SettingsFile.load(settings);
    }

    public static void save() {
        SettingsFile.save(settings);        
    }
    
    public static SettingsView getView(JFrame parent) {
        SettingsView view = new SettingsView(parent, "Data Crow Settings Panel", settings);
        view.setDisclaimer(DataCrow.baseDir + "icons_system/logo.jpg");
        view.setSize(new Dimension(875, 450));
        Image image = IconLibrary._icoSettings.getImage();
        view.setIconImage(image);
        image.flush(); 
        return view;
    }

    public static Object getValue(String key) {
        return settings.getValue(key);
    }
    
    public static Definitions getValueAsDefinitionGroup(String key) {
        return settings.getValueAsDefinitionGroup(key);
    }
    
    public static String[] getValueAsStringArray(String key) {
        return settings.getValueAsStringArray(key);
    }
    
    public static int[] getValueAsIntegerArray(String key) {
        return settings.getValueAsIntegerArray(key);
    }

    public static Font getValueAsFont(String key) {
        return settings.getValueAsFont(key);
    }    
    
    public static int getValueAsInt(String key) {
        return settings.getValueAsInt(key);
    }

    public static boolean getValueAsBoolean(String key) {
        return settings.getValueAsBoolean(key);
    }


    public static Color getValueAsColor(String key) {
        return settings.getValueAsColor(key);
    }
    
    public static Dimension getValueAsDimension(String key) {
        return settings.getValueAsDimension(key);
    }


    public static String getValueAsString(String key) {
        return settings.getValueAsString(key);
    }

    public static void setValue(String key, Object value) {
        settings.setValue(key, value);
    }

    public static void setStringAsValue(String key, String value) {
        settings.setStringAsValue(key, value);
    }    
    
    /**
     * Creates the settings group (with their hierarchy)
     */
    private void createGroups() {
        SettingsGroup generalGroup = new SettingsGroup(_General, "dc.Settings.GeneralSettings");
        SettingsGroup tableColorGroup = new SettingsGroup(_TableColors, "dc.Settings.TableColors");
        SettingsGroup backGround = new SettingsGroup(_BackGroundColor, "dc.Settings.BackGroundColor");
        SettingsGroup font = new SettingsGroup(_Font, "dc.Settings.Font");
        SettingsGroup http = new SettingsGroup(_HTTP, "dc.Settings.HttpSettings");
        SettingsGroup fileHandlers = new SettingsGroup(_FileHandlers, "dc.Settings.FileHandlers");
        
        tableColorGroup.addChild(new SettingsGroup(_OddRowColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_EvenRowColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_HeaderColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_RowSelectionColor, "dc.Settings.TableColors"));
        
        settings.addGroup(_General, generalGroup);
        settings.addGroup(_BackGroundColor, backGround);
        settings.addGroup(_HTTP, http);
        settings.addGroup(_TableColors, tableColorGroup);
        settings.addGroup(_Font, font);
        settings.addGroup(_FileHandlers, fileHandlers);
    }
    
    /**
     * Creates the settings and adds them to the specified group.
     * If a self-created component needs to be added, make sure this
     * components implements the ISettingsComponent class!
     * Otherwise the component is not supported.. The getValue and
     * setValue must correspond to the DataType of the setting.
     */
    private void createSettings() {
        settings.addSetting(_General, 
                new Setting(Repository.datatypes._INTEGER, 
                            Repository.settings.stVersionMajor, 
                            new Integer(Repository.system.versionMajor),
                            null,
                            "", 
                            "", 
                            false, 
                            false));   
        settings.addSetting(_General, 
                new Setting(Repository.datatypes._INTEGER, 
                            Repository.settings.stVersionMinor,
                            new Integer(Repository.system.versionMinor), 
                            null,
                            "", 
                            "", 
                            false, 
                            false));
        settings.addSetting(_General, 
                new Setting(Repository.datatypes._INTEGER, 
                            Repository.settings.stVersionBuild,
                            new Integer(Repository.system.versionBuild), 
                            null,
                            "", 
                            "", 
                            false, 
                            false));        
    	settings.addSetting(_BackGroundColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.settings.stColor,
                            new Color(230, 236, 230),
                            new DcColorSelector(true, 
                                Repository.settings.stColor),
                            "",                                     
                            DcResources.getText(Repository.labels.lblColor), 
                            false,                                  
                            true));                                 
    
    	settings.addSetting(_RowSelectionColor, 
                new Setting(Repository.datatypes._COLOR, 
                            Repository.settings.stRowSelectionColor,
                            new Color(192, 197, 214), 
                            new DcColorSelector(false,
                                Repository.settings.stRowSelectionColor),
                            "", 
                            DcResources.getText(Repository.labels.lblRowSelectionColor), 
                            false, 
                            true));         
    
    	settings.addSetting(_OddRowColor, 
                new Setting(Repository.datatypes._COLOR, 
                		    Repository.settings.stOddRowColor,
							new Color(224, 238, 227), 
                            new DcColorSelector(false,
                                Repository.settings.stOddRowColor),
						    "", 
                            DcResources.getText(Repository.labels.lblOddRowColor), 
                            false, 
                            true));

    	settings.addSetting(_EvenRowColor, 
                new Setting(Repository.datatypes._COLOR,
                		    Repository.settings.stEvenRowColor, 
                            new Color(247, 247, 239), 
                            new DcColorSelector(false,
                                Repository.settings.stEvenRowColor),
                            "",
							DcResources.getText(Repository.labels.lblEvenRowColor), 
                            false, 
                            true));
    
    	settings.addSetting(_HeaderColor, 
                new Setting(Repository.datatypes._COLOR, 
                            Repository.settings.stTableHeaderColor,
                            new Color(211, 220, 221), 
                            new DcColorSelector(false,
                                Repository.settings.stTableHeaderColor),
                            "", 
                            DcResources.getText(Repository.labels.lblTableHeaderColor), 
                            false, 
                            true));         
    	settings.addSetting(_General,
                new Setting(  Repository.datatypes._INTEGER,
                            Repository.settings.stMaximumSearchRes,
                            new Integer(100),
                            ComponentFactory._NUMBERFIELD,
                            DcResources.getText(Repository.tooltips.tpSearchRes),
                            DcResources.getText(Repository.labels.lblMaximumSearchRes),
                            true,
                            true));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stCheckUniqueness,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText(Repository.tooltips.tpUniqueness),
                            DcResources.getText(Repository.labels.lblUniqueness),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stCheckRequiredFields,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText(Repository.tooltips.tpRequiredFields),
                            DcResources.getText(Repository.labels.lblCheckRequiredFields),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stConnectionString,
                            "dc",
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stDatabaseDriver,
                            "org.hsqldb.jdbcDriver",
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowTableTooltip,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText(Repository.tooltips.tpShowTableTooltips),
                            DcResources.getText(Repository.labels.lblShowTableTooltips),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stModule,
                            new Integer(1),
                            null,
                            "",
                            "",
                            true,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stAutoLoadData,
                            Boolean.TRUE,
                            ComponentFactory._CHECKBOX,
                            DcResources.getText(Repository.tooltips.tpAutoLoadData),
                            DcResources.getText(Repository.labels.lblAutoLoadData),
                            false,
                            true));         
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowHiddenFieldsInForm,
                            Boolean.TRUE,
                            ComponentFactory._CHECKBOX,
                            DcResources.getText(Repository.tooltips.tpShowHiddenFieldsInForms),
                            DcResources.getText(Repository.labels.lblShowHiddenFieldsInForms),
                            false,
                            true));             
    	settings.addSetting(_Font,
                new Setting(Repository.datatypes._FONT,
                		    Repository.settings.stSystemFontNormal,
							new Font("Arial", Font.PLAIN, 10),
							ComponentFactory._FONTSELECTOR,
							DcResources.getText(Repository.tooltips.tpFont),
							DcResources.getText(Repository.labels.lblFontNormal),
							true,
							true));
    	settings.addSetting(_Font,
                new Setting(Repository.datatypes._FONT,
                		    Repository.settings.stSystemFontBold,
                            new Font("Arial", Font.PLAIN, 10),
							ComponentFactory._FONTSELECTOR,
							DcResources.getText(Repository.tooltips.tpFont),
							DcResources.getText(Repository.labels.lblFontBold),
							true,
							true));        
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stHTTPUserAgent,
                            "Mozilla/4.04 (Windows; I; Win95)",
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stProxyServerName,
                            "",
                            ComponentFactory._SHORTTEXTFIELD,
                            DcResources.getText(Repository.tooltips.tpProxyServerName),
                            DcResources.getText(Repository.labels.lblProxyServerName),
                            true,
                            true));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stProxyServerPort,
                            new Integer(0),
                            ComponentFactory._NUMBERFIELD,
                            DcResources.getText(Repository.tooltips.tpProxyServerPort),
                            DcResources.getText(Repository.labels.lblProxyServerPort),
                            true,
                            true));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stProxyUserName,
                            "",
                            ComponentFactory._SHORTTEXTFIELD,
                            DcResources.getText(Repository.tooltips.tpProxyUserName),
                            DcResources.getText(Repository.labels.lblProxyUserName),
                            true,
                            true));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stProxyPassword,
                            "",
                            ComponentFactory._PASSWORDFIELD,
                            DcResources.getText(Repository.tooltips.tpProxyPassword),
                            DcResources.getText(Repository.labels.lblProxyPassword),
                            true,
                            true));        
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stBrowserPath,
                            "",
                            ComponentFactory._FILEFIELD,
                            "",
                            DcResources.getText(Repository.labels.lblBrowserPath),
                            true,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSoftwareCatFormSize,
                            new Dimension(700, 600),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stBookCatFormSize,
                            new Dimension(700, 600),
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stExpertFormSize,
                            new Dimension(700, 600),
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stHelpFormSize,
                            new Dimension(700, 600),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stTabbedPaneSize,
                            new Dimension(741, 496),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSoftwareItemFormViewSize,
                            new Dimension(450, 550),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stBookItemFormViewSize,
                            new Dimension(450, 550),
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMusicItemFormViewSize,
                            new Dimension(450, 550),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMovieItemFormViewSize,
                            new Dimension(450, 550),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stWizardFormSize,
                            new Dimension(450, 550),
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stItemFormViewSize,
                            new Dimension(450, 550),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSoftwareCatItemFormViewSize,
                            new Dimension(407, 165),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stBookCatItemFormViewSize,
                            new Dimension(407, 165),
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMusicOrganizerViewSize,
                            new Dimension(300, 400),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stUpdateAllDialogSize,
                            new Dimension(600, 300),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMainViewSize,
                            new Dimension(865, 593),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stFreedbServer,
                            "freedb.freedb.org",
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stAmazonServer,
                            "http://xml.amazon.com",
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stFieldSettingsDialogSize,
                            new Dimension(500, 300),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stTextViewerSize,
                            new Dimension(500, 700),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stImdbSearchFormViewSize,
                            new Dimension(547, 469),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stAmazonSearchFormViewSize,
                            new Dimension(533, 495),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                         Repository.settings.stReportingDialogSize,
                         new Dimension(533, 490),
                         null,
                         "",
                         "",
                         false,
                         false));          
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                         Repository.settings.stModuleSelectDialogSize,
                         new Dimension(300, 300),
                         null,
                         "",
                         "",
                         false,
                         false));       
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stFreeDBSearchFormViewSize,
                            new Dimension(533, 495),
                            null,
                            "",
                            "",
                            false,
                            false));         
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stImportAudioCDDialogSize,
                            new Dimension(542, 680),
                            null,
                            "",
                            "",
                            false,
                            false));         
    	settings.addSetting(_General,
               new Setting(Repository.datatypes._DIMENSION,
                           Repository.settings.stImportCDDialogSize,
                           new Dimension(550, 400),
                           null,
                           "",
                           "",
                           false,
                           false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowModuleList,
                            new Boolean(true),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stODBOverwriteValues,
                            new Boolean(false),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stBroadband,
                            new Boolean(true),
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stLastDirectoryUsed,
                            "",
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stImdbServer,
                            "http://www.imdb.com",
                            null,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowMenuBarLabels,
                            Boolean.TRUE,
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stUpdateAllSelectedItemsOnly,
                            Boolean.TRUE,
                            null,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowQuickView,
                            Boolean.TRUE,
                            null,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stDeleteImageFileAfterImport,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblDeleteImageFile)),
                            DcResources.getText(Repository.tooltips.tpDeleteImageFile),
                            "",
                            false,
                            true));     
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stQuickViewSize,
                            "25%",
                            null,
                            "",
                            "",
                            false,
                            false));     
        settings.addSetting(_FileHandlers,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stProgramDefinitions,
                            new ProgramDefinitions(),
                            ComponentFactory.getProgramDefinitionField(),
                            DcResources.getText(Repository.tooltips.tpProgramDefinitions),
                            "",
                            false,
                            true));   
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stResourcesEditorViewSize,
                            new Dimension(542, 680),
                            null,
                            "",
                            "",
                            false,
                            false));         
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stBackupDialogSize,
                            new Dimension(500, 500),
                            null,
                            "",
                            "",
                            false,
                            false));        
        
        setSoftwareDefaults();
        setSoftwareCategoryDefaults();
        setAudioCDDefaults();
        setAudioTrackDefaults();
        setBookDefaults();
        setBookCategoryDefaults();
        setMovieDefaults();
        setMusicFileDefaults();
        setGenericObjectDefaults();
    }
    
    private void setGenericObjectDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(GenericObject._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._SYS_MODULE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U5_USER_NUMERIC2, null, false, false, false));        
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsGeneric,
                            definitions, null, "", "", false, false));  
    }
    
    private void setSoftwareCategoryDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(SoftwareCategory._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(SoftwareCategory._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(SoftwareCategory._B_ICON, null, true, false, true));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftwareCategory,
                            definitions, null, "", "", false, false));         
    }      
    
    private void setBookCategoryDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(BookCategory._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(BookCategory._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(BookCategory._B_ICON, null, true, false, true));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsBookCategory,
                            definitions, null, "", "", false, false));         
    }      
    
    private void setBookDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Book._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Book._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._F_PUBLISHER, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._G_AUTHOR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._H_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._I_CATEGORY, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._J_ISBN, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._K_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._E_RATING, null, true, false, true));
//        definitions.addDefinition(new DcFieldDefinition(Book._SYS_MODULE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U5_USER_NUMERIC2, null, false, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsBook,
                            definitions, null, "", "", false, false));         
    }      
    
    private void setAudioTrackDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._F_TRACKNUMBER, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._A_TITLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._B_DESCRIPTION, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._C_YEAR, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._D_LOCATION, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._E_RATING, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._G_LYRICS, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._H_PLAYLENGTH, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._I_ALBUM, null, true, false, false));
//        definitions.addDefinition(new DcFieldDefinition(AudioTrack._SYS_MODULE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U5_USER_NUMERIC2, null, false, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsAudioTrack,
                            definitions, null, "", "", false, false));         
    }      
    
    private void setAudioCDDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(AudioCD._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._F_ARTIST, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._G_GENRE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._H_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._I_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._J_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._K_DISCID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._E_RATING, null, true, false, true));
//        definitions.addDefinition(new DcFieldDefinition(AudioCD._SYS_MODULE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U5_USER_NUMERIC2, null, false, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsAudioCD,
                            definitions, null, "", "", false, false));         
    }     
    
    private void setMovieDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Movie._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._F_TITLE_LOCAL, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._G_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._H_GENRE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._I_ACTORS, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._J_DIRECTOR, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._K_COUNTRY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._L_PLAYLENGTH, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._M_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._N_VIDEOCODEC, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._O_AUDIOCODEC, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._P_WIDTH, null, true, false, false));        
        definitions.addDefinition(new DcFieldDefinition(Movie._Q_HEIGHT, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._R_FPS, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._S_FRAMES, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._T_AUDIOBITRATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U_AUDIOSAMPLINGRATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._V_AUDIOCHANNEL, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._W_VIDEOBITRATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._X_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._Y_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._Z_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._0_LANGUAGE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._1_AUDIOLANGUAGE, null, true, false, false));        
        definitions.addDefinition(new DcFieldDefinition(Movie._2_SUBTITLELANGUAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._3_CERTIFICATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._4_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._E_RATING, null, true, false, true));
//        definitions.addDefinition(new DcFieldDefinition(Movie._SYS_MODULE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U5_USER_NUMERIC2, null, false, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMovie,
                            definitions, null, "", "", false, false));         
    }    
    
    private void setMusicFileDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(MusicFile._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._F_ARTIST, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._G_ALBUM, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._H_GENRE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._I_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._J_PLAYLENGTH, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._K_QUALITY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._L_ENCODING, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._M_LYRIC, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._N_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._E_RATING, null, true, false, true));
//        definitions.addDefinition(new DcFieldDefinition(MusicFile._SYS_MODULE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicFile._U5_USER_NUMERIC2, null, false, false, false));
        
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMusic,
                            definitions, null, "", "", false, false));         
    }    
    
    private void setSoftwareDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Software._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Software._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._F_DEVELOPER, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._G_PUBLISHER, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._H_PLATFORM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._I_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._J_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._K_TYPE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._L_MULTI, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._M_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._N_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._O_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._P_SCREENSHOTONE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._Q_SCREENSHOTTWO, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._R_SCREENSHOTTHREE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._S_SERIALKEY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._T_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._E_RATING, null, true, false, true));
//        definitions.addDefinition(new DcFieldDefinition(Software._SYS_MODULE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U5_USER_NUMERIC2, null, false, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftware,
                            definitions, null, "", "", false, false));         
    }
}
