/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.jflexsett.settings;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;

/**
 * Reads and saves data to the Settings File
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class SettingsFile {

    /**
     * Save the settings file
     */
    public static void save(Settings settings) {
        Properties properties = new Properties();
        
        Collection c = settings.getSettings();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            Setting setting = (Setting) iter.next();
            properties.setProperty(setting.getKey(), setting.getValueAsString());
        }

        try {
            FileOutputStream fos = new FileOutputStream(settings.getSettingsFile());
            properties.store(fos, "");
            fos.close();
        } catch (Exception exp) {
            exp.printStackTrace();    
        } 
    }

    /**
     * Loads the settings file
     */
    public static void load(Settings settings) {
        Properties properties = new Properties();
        
        try {
            FileInputStream fis = new FileInputStream(settings.getSettingsFile());
            properties.load(fis);
            fis.close();
        } catch (Exception exp) {} // use default settings
        
        Collection c = settings.getSettings();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            Setting setting = (Setting) iter.next();
            if (properties.getProperty(setting.getKey()) != null) {
                String sValue = properties.getProperty(setting.getKey()).toString();
                settings.setStringAsValue(setting.getKey(), sValue);
            }
        }        
    }
}
